'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''


class Base():
    """This is a Base class.
    This class in inherited in all classes
    of the Network package.
    Inheriting this class avoids the problem of
    calling object with **args/**kwargs while
    using super method (while invoking
    the methods of the Network package using
    an instance of Network class)
    Base class for Network package """
    def __init__(self, *args, **kwargs):
        """
        Constructor
        """



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

"""Warrior Network diagnostics module """
import re
from warriornetwork.ClassUtils.WNetwork.base_class import Base
from warrior.Framework.Utils.testcase_Utils import pNote


class Diag(Base):
    """Warrior Diagnostics class """

    def __init__(self, *args, **kwargs):
        """ Constructor """
        super(Diag, self).__init__(*args, **kwargs)

    @staticmethod
    def ping_from_remotehost(session_object, ip_type,
                             dest_address, prompt, count):
        """ping  to dest_system from remote host

        :Arguments:
            1. session_object(string)  = expect session object
            2. command(string) = command to be executed
            3. ip_type = ip/ipv6/dns
            4. dest_address(string) = ip or dns name
            3. prompt(string) = prompt

        :Returns:
            1. bool (True/False)

        """

        ping_cmd = "ping"
        if ip_type == "ipv6":
            ping_cmd = "ping6"

        command = ping_cmd + " -c {} {}".format(count, dest_address)

        _, output = session_object.send_command(".*", prompt, command)
        if " 0% packet loss" in output or "alive" in output:
            pNote("ping successfully completed")
            status = True
        else:
            pNote("ping command failed")
            status = False

        return status

    @staticmethod
    def traceroute_from_remotehost(session_object, ip_type, dest_address,
                                   prompt):
        """traceroute to dest_system from remote host

        :Arguments:
            1. session_object(string)  = expect session object
            2. command(string) = command to be executed
            3. ip_type = ip/ipv6/dns
            4. dest_address(string) = ip or dns name
            3. prompt(string) = prompt

        :Returns:
            1. bool (True/False)

        """
        traceroute_cmd = "traceroute"
        if ip_type == "ipv6":
            traceroute_cmd = "traceroute6"

        command = traceroute_cmd + " {}".format(dest_address)

        _, output = session_object.send_command(".*", prompt, command)
        if not isinstance(output, str):
            pNote("traceroute command returned invalid value", "error")
            return False

        output_list = output.split("\n")
        for output in output_list:
            match = re.search(r'^ *(\d+) .*?'+dest_address, output)
            if match:
                break
        if match:
            pNote("Traceroute completed in {} hop".format(match.group(1)))
            status = True
        else:
            status = False
            pNote("Traceroute failed", "error")

        return status



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

"""SSH utility module using the python Paramiko module"""
#!/usr/bin/env python

import os
import time
from time import sleep
from warrior.Framework.Utils import testcase_Utils, data_Utils
from warrior.Framework.Utils.print_Utils import print_info, print_error, print_debug, print_exception

class SSHComm(object):
    """SSHComm class which has methods
       related to SSH communication using paramiko"""

    def __init__(self, target, port=22, uid='', pid='', logfile=None):
        """
        Constructor
        """
        self.param = None
        self.import_param()
        self.sshobj = self.param.SSHClient()
        self.target = target
        self.uid = uid
        self.pid = pid
        self.logfile = logfile
        self.port = port

    def import_param(self):
        """Import the paramiko module """
        try:
            import paramiko
        except ImportError:
            print_error("Paramiko module is not installed"\
                       "Please install requests module to"\
                       "perform any activities related to SSH Communication")
        else:
            self.param = paramiko

    def ssh_con(self, retries=1, interval=1, timeout=60, verify_keys=False,
                invoke_shell=False):
        """Connects to the host using ssh object

        :Arguments:
            1. retries =  No of attempts before returning fail
            2. interval = Time to wait before the next retry
            3. timeout = wait for response
            4. verify_keys = Verify the host entry is available in host key

        :Returns:
            1. status(bool)= True / False

        """
        for attempt in range(retries):
            print_debug("Attempt{} connecting to {}".format(attempt+1,
                                                            self.target))
            try:
                if not verify_keys:
                    self.sshobj.set_missing_host_key_policy(\
                                                self.param.AutoAddPolicy())
                self.sshobj.connect(self.target, self.port, self.uid,
                                    self.pid, timeout=timeout,
                                    look_for_keys=verify_keys)
                if invoke_shell:
                    print_info("Opening shell for {}".format(self.sshobj))
                    self.sshobj.invoke_shell()

                if self.logfile is not None:
                    self.log = open(self.logfile, 'w')
            except self.param.SSHException:
                print_error(" ! could not connect to %s...check logs" % self.target)
                return False
            except Exception as err:
                print_error("Login failed {0}".format(str(err)))
                sleep(interval)
                continue
            else:
                print_info("Connected to the host")
                return True
        return False

    def ssh_close(self):
        """Close the SSH connection object

        :Returns:
            1. status(bool)= True / False

        """
        try:
            print_info("Closing SSH connection")
            self.sshobj.close()
            print_info("SSH Connection closed")

        except Exception:
            print_error("Error occured while closing ssh session")
            return False

        else:
            return True

    def ssh_exec(self, command, timeout=None, get_pty=True):
        """Execution of the  command on the remote host

        :Arguments:
           1.command = command to be executed
           2.timeout = wait for response
           3.get_pty = request a pseudo-terminal

        :Returns:
           1. out = command output
           2. stderr = command's error stream object
        """
        print_info("Executing command : %s" % command)
        out = ""
        stdin, stdout, stderr = self.sshobj.exec_command(command,
                                                         timeout=timeout,
                                                         get_pty=get_pty)
        stdin.close()
        if self.logfile is not None:
            self.log.write("*"*20)
            self.log.write("\n")
            self.log.write("Executing command :%s\n" % command)
            self.log.write("Output:\n")

        for line in iter(lambda: stdout.readline(2048), ""):
            print_info(line)
            out = out + line
            if self.logfile:
                self.log.write(line)
                self.log.flush()

        return out, stderr

    def get_response(self, command, timeout=None):
        """Execute a command using the paramiko SSH object and returns
           the response string

          :Arguments:
               1. command - commnad to be executed
               2. timeout - wait for response

          :Returns:
              1. status(bool)= True / False
              2. out/err = response string
        """
        out, stderr = self.ssh_exec(command, timeout)
        err = stderr.read()
        if out:
            return True, out
        else:
            print_error("Error:", err)
            return False, err

    def verify_response(self, command, exp_string, fail_resp, timeout=None):
        """Execute a command using the paramiko SSH object and returns
           the response string

           :Arguments:
               1. command - command to be executed
               2. exp_string - String to be expected
               3. fail_resp - Failure response to be expected
               4. timeout - wait for response

          :Returns:
               1. status(bool)= True / False
               2. out/err = response string
        """
        out, stderr = self.ssh_exec(command, timeout)
        err = stderr.read()
        if out and ((exp_string in out) and (fail_resp not in out)):
            print_info("Output:", out)
            return True, out
        elif out and ((exp_string not in out) or (fail_resp in out)):
            print_info("Error:", out)
            return False, out
        else:
            print_error("Error:", err)
            return False, err

    def copy_file(self, remotepath, localpath, filename):
        """Get the file from remotepath

        :Argument:
             1. remotepath - File path in the remote server
             2. localpath - Local path to save the file
             3. filename - Filename of the file to be tranferred

        :Returns:
             1. status(bool)= True / False
        """
        sftpobj = self.sshobj.open_sftp()
        print_info("Copying {} from {} to {}".format(filename, remotepath,
                                                     localpath))
        if filename:
            remotepath = os.path.join(remotepath, filename)
            localpath = os.path.join(localpath, filename)
        sftpobj.get(remotepath, localpath)
        if os.path.isfile(localpath):
            return True
        else:
            return False

    def send_testdata_cmds(self, testdatafile, **args):
        """
        - Parses the testdata file and
          - gets the command details for rows marked execute=yes (or)
          - gets the command details for rows marked execute=yes and row=str_rownum (or)
          - gets the command details for rows marked execute=yes and title=strtitle
        - Sends the obtained commands to the paramiko session.
        - If the commands have verification attribute set,
        then verifies the verification text for presence/absence as defined
        in the respective found attribute in the testdatfile.

        :Arguments:
            1. testdatafile = the xml file where command details are available
            2. str_rownum = row number of testdata command block to be searched for
               in the testdata file
            3. strtitle = title of testdata command block title to be searched
               for in the testdata file
        :Returns:
            1. finalresult = boolean
        """
        finalresult = True
        varconfigfile = args.get('varconfigfile', None)
        title = args.get('title', None)
        row = args.get('row', None)
        details_dict = data_Utils.get_command_details_from_testdata(testdatafile, varconfigfile, title=title, row=row)
        command_list = details_dict["command_list"]
        stepdesc = "Send the following commands: %s" %command_list
        testcase_Utils.pNote(stepdesc)
        if command_list == False:
            finalresult = False
            command_list = []
        intsize = len(command_list)
        # Send Commands
        for i in range(0, intsize):
            command = details_dict["command_list"][i]
            startprompt = details_dict["startprompt_list"][i]
            endprompt = details_dict["endprompt_list"][i]
            cmd_timeout = details_dict["timeout_list"][i]
            retry = details_dict["retry_list"][i]
            retry_timer = details_dict["retry_timer_list"][i]
            retry_onmatch = details_dict["retry_onmatch_list"][i]
            retry_count = details_dict["retry_count_list"][i]
            sleeptime = details_dict["sleeptime_list"][i]
            verify_text_list = details_dict["verify_text_list"][i]
            verify_context_list = details_dict["verify_context_list"][i]
            sleep = {None: 0, "":0, "none":0}.get(str(sleeptime).lower(), str(sleeptime))
            sleep = int(sleep)
            print_debug("\n")
            print_debug(">>>")
            testcase_Utils.pNote("Command #{0}\t: {1}".format(str(i+1), command))
            testcase_Utils.pNote("startprompt\t: {0}".format(startprompt))
            testcase_Utils.pNote("endprompt\t: {0}".format(endprompt))
            testcase_Utils.pNote("sleeptime\t: {0}".format(sleep))        
            result, response = self.get_response(command_list[i])
            if result and result is not 'ERROR':
                if verify_text_list is not None:
                    result = data_Utils.verify_cmd_response(verify_text_list, verify_context_list,
                                                            command, response, varconfigfile)
            command_status = {True: "Pass", False:"Fail", "ERROR":"ERROR"}.get(result)
            print_info("Command status: {0}".format(command_status))
            print_debug("<<<")
            if sleep > 0:
                testcase_Utils.pNote("Sleep time of '{0} seconds' requested post command execution".format(sleep))
                time.sleep(sleep)        
            if result == "ERROR" or finalresult == "ERROR":
                result = "ERROR"
                finalresult = "ERROR"        
            finalresult = finalresult and result
        return finalresult

    def download_remote_file_sftp(self, remotepath, localpath, filename, \
                                remotehost, username, password, port=22):
        """Downloads a remote file from the remote server to
        the localpath using SFTP

        :Arguments:
            1. remotepath (string) = the remote path to download the file from remotehost
            2. localpath (string) = the local path on the local host
            3. filename (string) = name of the remote file to download
            4. remotehost (string) = remote host
            5. username (string) = remote login username
            6. password (string) = remote login password

        :Returns:
            1. status(bool)
        """
        status = False
        transport = self.param.Transport((remotehost, port))
        transport.connect(username=username, password=password)
        sftp = self.param.SFTPClient.from_transport(transport)

        if filename:
            try:
                sftp.get(remotepath + filename, localpath + filename)
                if os.path.isfile(localpath + filename):
                    print_info("Downloading file from remote server successful")
                    status = True
                else:
                    print_info("Downloading file from remote server failed")
            except Exception as exception:
                print_exception(exception)

            finally:
                sftp.close()
                transport.close()

        return status

    def invoke_shell(self, term='vt100'):
        """Opens a terminal for the SSH session
           :Argument:
             1. term = Type of terminal, default:vt100

           :Returns:
             True: if successful
             False: if unsuccessful
        """
        try:
            print_info("Opening shell...")
            self.sshobj.invoke_shell(term=term)
            return True

        except self.param.SSHException:
            print_error("Invoke shell Unsuccessful")
            return False

    def connect_target_via_host(self, target, user, auth, invoke_shell=False,
                                log=None):
        """Forward the SSH connection to another target client
           :Argument:
             1. target(string) - Name/ip target machine to be connected
             2. user - username to connect
             3. auth - password for the user
             4. invoke_shell - open shell for passing commands

           :Returns:
             1. target_session - Session object for target connection
             2. status(bool)= True / False

        """
        target_session = SSHComm(target, uid=user, pid=auth, logfile=log)
        status = False
        try:
            ssh_transport = self.sshobj.get_transport()


            channel = ssh_transport.open_channel("direct-tcpip", (target, 22),
                                                 ('localhost', 0))
            print_info("Connecting to target: {}".format(target))
            target_session.sshobj.set_missing_host_key_policy(\
                                       self.param.AutoAddPolicy())
            target_session.sshobj.connect(target, port=22, username=user,
                                          password=auth, sock=channel)
            if target_session.logfile:
                target_session.log = open(self.logfile, 'w')
            print_info("Connection to target: {}successful".format(target))
            if invoke_shell:
                print_info("Opening shell for {}".format(target))
                target_session.invoke_shell()

            status = True

        except self.param.SSHException as exception:
            print_exception(exception)

        except Exception as exception:
            print_exception(exception)

        return target_session, status



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

"""Warrior Network File operations module """

import re
from warriornetwork.ClassUtils.WNetwork.base_class import Base
from warrior.Framework.Utils import file_Utils
from warriornetwork.Utils import cli_Utils
from warrior.Framework.Utils.testcase_Utils import pNote


class FileOps(Base):
    """Warrior File operations class """

    def __init__(self, *args, **kwargs):
        """ Constructor """
        super(FileOps, self).__init__(*args, **kwargs)

    @staticmethod
    def ftp_put_from_remote(session_object, filename):
        """For ftping the file to destination
        Assumes a ftp session is already present

        :Arguments:
            1. session_object-session object to be used
            2. filename(string) - filename string

        :Returns
            1. status(bool) - True/False

        """
        _, output = session_object.send_command(".*", ">", "put {}".format(filename))
        if output.find("226 Transfer complete"):
            status = True
            pNote("successfully put file from dest")
        else:
            status = False

        return status

    @staticmethod
    def sftp_get_from_remote(session_object, filename, filedir):
        """For Fetching the file from destination
        Assumes a sftp session is already present

        :Arguments:
            1. session_object-session object to be used
            2. filename(string) - filename string
            3. filedir(string) - directory path of the file

        :Returns
            1. status(bool) - True/False

        """
        _, output = session_object.send_command(".*", ">", "get {}".format(filename))
        if output.find("Fetching {0} to {1}".format(filedir + "/" + filename, filename)):
            status = True
            pNote("successfully got file from dest")
        else:
            status = False

        return status

    @staticmethod
    def sftp_put_from_remote(session_object, filename, filedir):
        """For sftping the file to destination
        Assumes a sftp session is already present

        :Arguments:
            1. session_object-session object to be used
            2. filename(string) - filename string
            3. filedir(string) - directory path of the file

        :Returns
            1. status(bool) - True/False

         """
        _, output = session_object.send_command(".*", ">", "put {}".format(filename))
        if output.find("Uploading {0} to {1}".format(filename, filedir+"/"+filename)):
            status = True
            pNote("successfully put file from dest")
        else:
            status = False

        return status

    @staticmethod
    def ftp_get_from_remote(session_object, filename):
        """For fetching the file from destination
        Assumes a ftp session is already present

        :Arguments:
            1. session_object-session object to be used
            2. filename(string) - filename string

        :Returns
            1. status(bool) - True/False

         """

        # To check Fetching from remote host
        _, output = session_object.send_command(".*", ">", "get {}".format(filename))
        if output.find("226 Transfer complete"):
            status = True
            pNote("successfully got file from dest")
        else:
            status = False

        return status

    @staticmethod
    def get_file_size(session_object, prompt, filename, session=None):
        """
        To get file size for a given file
        This assumes you are in the correct directory
        This also handles file size in ftp and sftp prompt

        :Arguments:
            1. session_object = session object to be used
            2. prompt(string) = prompt expected
            2. filename(string) = filename string
            3. session(string) = ftp/sftp/others

        :Returns
            1. status(bool) = True/False

         """
        if session == "ftp":

            command = "ls {} ".format(filename)
            _, output = session_object.send_command(".*", ">", command)
            size1 = ([line for line in output.split("\n")
                      if filename in line][1].split(" "))
            size = [line for line in size1 if line != ""][4]

        elif session == "sftp":

            command = "ls -l {} ".format(filename)
            _, output = session_object.send_command(".*", ">", command)
            size1 = output.split("\n")[1].split(" ")
            size = [line for line in size1 if line != ""][4]
        else:

            command = "ls -l {} | awk '{{print $5}}'".format(filename)
            _, output = session_object.send_command(".*", prompt, command)
            # to remove discrepancy between solaris and linux
            output_list = [output.replace('\r', '')
                           for output in output.split("\n")
                           if command not in output]
            size = output_list[0]

        pNote("File:{0} size found to be:{1}".format(filename, str(size)))
        return size

    @staticmethod
    def file_exists_on_remote(session_object, prompt, filename):
        """
        To check file exists
        This assumes you are in the correct directory

        :Arguments:
            1. session_object = session object to be used
            2. prompt(string) = prompt expected
            3. filename(string) = filename string

        :Returns
            1. status(bool) = True/False

        """
        status = False
        command = "ls -l {}".format(filename)
        _, output = session_object.send_command(".*", prompt, command)
        if output.find("No such file or directory") >= 0:
            pNote("File:{} not  found".format(filename), "error")
        else:
            status = True

        return status

    @staticmethod
    def start_ftp_on_remote(session_object, command, username, password):
        """ To start ftp connection on remote machine

        :Arguments:
            1. session_object = session object to be used
            2. command(string) = ftp command
            2. username(string) = username
            3. password(string) = password

        :Returns
            1. status(bool) = True/False

        """
        session_object.send_command(".*", ":", command)
        session_object.send_command(".*", ":", username)
        status, response = session_object.send_command(".*", ">", password)
        if status and "Login successful" in response:
            status = True
            pNote("ftp session established successfully")
        else:
            status = False
            pNote("ftp session cannot be established", "error")
        return status

    @staticmethod
    def start_sftp_on_remote(session_object, command, password, prompt):
        """ To start sftp connection on remote machine

        :Arguments:
            1. session_object = session object to be used
            2. command(string) = sftp command
            3. username(string) = username
            4. password(string) = password
            5. prompt(string) = prompt in source system

        :Returns
            1. status(bool) = True/False

        """
        status, response = session_object.send_command(".*", "password:|>", command)

        if re.search('.*(?i)remote host identification has changed.*', response):
            line_del = re.search('Offending key in .*?:(\d+)', response).group(1)

            ssh_key_remove = "sed '"+line_del+"d' ~/.ssh/known_hosts> /tmp/temp "\
                             "&& mv -f /tmp/temp  ~/.ssh/known_hosts"

            status, _ = session_object.send_command(".*", prompt, ssh_key_remove)

            status, response = session_object.send_command(".*", "password:|>", command)
        if re.search('(yes/no)', response):
            status, _ = session_object.send_command(".*", '.*(?i)password.*', 'yes')
            if status is not True:
                status, response = session_object.send_command(".*", "password:|>", command)

        status, _ = session_object.send_command(".*", ">", password)
        if status is True:
            pNote("sftp session established successfully")
        else:
            status = False
            pNote("sftp session cannot be established", "error")
        return status

    @classmethod
    def ftp_from_remotehost(cls, session_object, ip_type, ftp_operation,
                            filepath, prompt, dest_address, username, password,
                            filepath_dest):
        """ftp(test both put and get).This keyword can be used to transfer
        file from source _system to destination system or vice versa.
        It checks the size after transfer in both get and put.

        :Arguments:
            1. session_object(string)  = name of the Linux machine on which\
                                      to execute
            2. ip_type(string) = iptype of the dest system through \
                                 which it needs to be connected.
                                 needs to be one of \
                                 (ip/ipv4/dns/lmp_ip/lmp_ipv6).It has to be \
                                 present in the input data file.
            3. ftp_operation(string) = get/put/both
            4. filepath(string) = file with filepath in source\
                                  system(used for put)
            5. prompt(string)  = prompt of the source system
            6. dest_address(string) = ipv4 address or defaulted to lcn ip
            7. username(string) = username of the dest system
            8. password(string) = password of the dest system
            9. filepath_dest(string) = file with filepath in destination\
                               system(used for get)
        :Returns:
            1. bool (True/False)

        """
        put = True if ("put" in ftp_operation or "both" in ftp_operation) \
            else False
        get = True if ("get" in ftp_operation or "both" in ftp_operation) \
            else False

        status_get = False if get else True
        status_put = False if put else True

        ftp_cmd = "ftp"
        if ip_type == "ipv6":
            ftp_cmd = "ftp6"

        filedir, filename = (file_Utils.getDirName(filepath),
                             file_Utils.getFileName(filepath))
        filedir_dest, filename_dest = (file_Utils.getDirName(filepath_dest),
                                       file_Utils.getFileName(filepath_dest))

        command = ftp_cmd + " [{}]".format(dest_address)
        # move into the path
        session_object.send_command(".*", prompt, "cd {}".format(filedir))

        # check whether file is available
        putfilepresent = cls.file_exists_on_remote(session_object, prompt,
                                                   filename)
        if putfilepresent:
            # check the put file size
            put_file_size = cls.get_file_size(session_object, prompt, filename)

        else:
            pNote("Specified put file:{} not found".format(filename), "error")

        # Starting the ftp connection
        status = cls.start_ftp_on_remote(session_object, command,
                                         username, password)
        if status:
            session_object.send_command(".*", ">", "cd {}".format(filedir_dest))

            if putfilepresent and put:
                status_put = cls.ftp_put_from_remote(session_object, filename)
                put_file_size_transf = cls.get_file_size(session_object, ">",
                                                         filename,
                                                         session="ftp")
                if status_put:
                    pNote("Actual put file size:{0} Transferred put file size"
                          ":{1}".format(str(put_file_size), str(put_file_size_transf)))
                    if str(put_file_size) == str(put_file_size_transf):
                        status_put = True
                        pNote("Transferred put file size matches the origin")
                    else:
                        status_put = False
                        pNote("Transferred put file size does not"
                              "match the origin")

            if get:
                # check whether file is available
                getfilepresent = cls.file_exists_on_remote(session_object, ">",
                                                           filename_dest)

                if getfilepresent:
                    gfile_size = cls.get_file_size(session_object, ">",
                                                   filename_dest,
                                                   session="ftp")
                    status_get = cls.ftp_get_from_remote(session_object,
                                                         filename_dest)

                    # exiting the ftp terminal
                    session_object.send_command(".*", prompt, "exit")

                    if status_get:
                        # check the get file size after transfer
                        gfile_size_transf = cls.get_file_size(session_object,
                                                              prompt,
                                                              filename_dest)

                        pNote("Actual get file size:{0} Transferred get file"
                              "size:{1}".format(str(gfile_size), str(gfile_size_transf)))
                        if str(gfile_size) == str(gfile_size_transf):
                            status_get = True
                            pNote("Transferred get file size matches the origin")
                        else:
                            status_get = False
                            pNote("Transferred get file size matches the origin")
                else:
                    pNote("Specified get file:{} not found".format(filename_dest),
                          "error")

            else:
                session_object.send_command(".*", prompt, "exit")

            status = status_get and status_put
        else:
            status = False
        return status

    @classmethod
    def sftp_from_remotehost(cls, session_object, ip_type, sftp_operation, port,
                             filepath, prompt, dest_address, username,
                             password, filepath_dest):
        """sftp(test both put and get).This keyword can be used to transfer
        file from source _system to destination system or vice versa.
        It checks the size after transfer in both get and put.

        :Arguments:
            1. session_object(string)  = name of the Linux machine on which\
                                      to execute
            2. ip_type(string) = iptype of the dest system through \
                                 which it needs to be connected.
                                 needs to be one of \
                                 (ip/ipv4/dns/lmp_ip/lmp_ipv6).It has to be \
                                 present in the input data file.
            3. sftp_operation(string) = get/put/both
            4. port(string) = source port
            5. filepath(string) = file with filepath in source\
                                  system(used for put)
            6. prompt(string)  = prompt of the source system
            7. dest_address(string) = ipv4 address or defaulted to lcn ip
            8. username(string) = username of the dest system
            9. password(string) = password of the dest system
            9. filepath_dest(string) = file with filepath in destination\
                               system(used for get)
        :Returns:
            1. bool (True/False)

        """
        put = True if ("put" in sftp_operation or "both" in sftp_operation) \
            else False
        get = True if ("get" in sftp_operation or "both" in sftp_operation) \
            else False

        status_get = False if get else True
        status_put = False if put else True

        filedir, filename = (file_Utils.getDirName(filepath),
                             file_Utils.getFileName(filepath))
        filedir_dest, filename_dest = (file_Utils.getDirName(filepath_dest),
                                       file_Utils.getFileName(filepath_dest))

        sftp_cmd = "sftp"
        if ip_type == "ipv6":
            sftp_cmd = "sftp6"

        # move into the path
        session_object.send_command(".*", prompt, "cd {}".format(filedir))
        command = sftp_cmd + "  -oPort={0} {1}@{2}".format(port, username,
                                                           dest_address)

        # check whether file is available
        putfilepresent = cls.file_exists_on_remote(session_object, prompt,
                                                   filename)
        if putfilepresent:
            # check the put file size
            put_file_size = cls.get_file_size(session_object, prompt, filename)

        else:
            pNote("Specified put file:{} not found".format(filename), "error")

        # Starting the sftp connection
        status = cls.start_sftp_on_remote(session_object, command,
                                          password, prompt)
        if status:
            session_object.send_command(".*", ">", "cd {}".format(filedir_dest))

            if putfilepresent and put:
                status_put = cls.sftp_put_from_remote(session_object,
                                                      filename, filedir)
                put_file_size_transf = cls.get_file_size(session_object, ">",
                                                         filename,
                                                         session="sftp")
                if status_put:
                    pNote("Actual put file size:{0} Transferred put file size"
                          ":{1}".format(str(put_file_size), str(put_file_size_transf)))
                    if str(put_file_size) == str(put_file_size_transf):
                        status_put = True
                        pNote("Transferred put file size matches the origin")
                    else:
                        status_put = False
                        pNote("Transferred put file size does not"
                              "match the origin")

            if get:
                # check whether file is available
                getfilepresent = cls.file_exists_on_remote(session_object, ">",
                                                           filename_dest)

                if getfilepresent:
                    gfile_size = cls.get_file_size(session_object, ">",
                                                   filename_dest,
                                                   session="sftp")
                    status_get = cls.sftp_get_from_remote(session_object,
                                                          filename_dest,
                                                          filedir_dest)

                    # exiting the sftp terminal
                    session_object.send_command(".*", prompt, "exit")

                    if status_get:
                        # check the get file size after transfer
                        gfile_size_transf = cls.get_file_size(session_object,
                                                              prompt,
                                                              filename_dest)

                        pNote("Actual get file size:{0} Transferred get file"
                              "size:{1}".format(str(gfile_size), str(gfile_size_transf)))
                        if str(gfile_size) == str(gfile_size_transf):
                            status_get = True
                            pNote("Transferred get file size matches the origin")
                        else:
                            status_get = False
                            pNote("Transferred get file size matches the origin")
                else:
                    pNote("Specified get file:{} not found".format(filename_dest),
                          "error")

            else:
                session_object.send_command(".*", prompt, "exit")

            status = status_get and status_put
        else:
            status = False
        return status




'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''


from warriornetwork.ClassUtils.WNetwork.base_class import Base
from warrior.Framework.Utils.print_Utils import print_error

class Connection(Base):
    """Warrior Network connectivity module
       Warrior connectivity class """
    def __init__(self, *args, **kwargs):
        """ Constructor """
        super(Connection, self).__init__(*args, **kwargs)

    def connect(self):
        """ connect """

    def connect_ssh(self):
        """ connect ssh """
        print_error("ssh connection")

    def connect_telnet(self):
        """ connect telnet """



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
import os
from warrior.Framework.Utils.print_Utils import print_info, print_warning
from warriornetwork.ClassUtils import WNetwork, ssh_utils_class
from warrior.WarriorCore.Classes.warmock_class import mocked
from warrior.WarriorCore.Classes.war_cli_class import WarriorCliClass

try:
    import pexpect
except ImportError:
    print_info("{0}: pexpect module is not installed".format(os.path.abspath(__file__)))
    print_info("Warrior framework by default uses pexpect for all cli related activites")
    print_info("All default methods/functions that use cli will fail"
               "without pexpect module. Users can however create"
               "their own custom libraries for cli interaction \n")

def pexpect_spawn_with_env(pexpect_obj, command, timeout, escape=False, env=None):
    """ spawn a pexpect object with environment variable """

    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    child = wc_obj.pexpect_spawn_with_env(pexpect_obj, command, timeout,
                                          escape, env)

    return child


@mocked
def connect_ssh(ip, port="22", username="", password="", logfile=None, timeout=60,
                prompt=".*(%|#|\$)", conn_options="", custom_keystroke="", escape="", **kwargs):
    """
    Initiates SSH connection via a specific port. Creates log file.
    :Arguments:
        1. ip = destination ip
        2. port(string) = telnet port
        3. username(string) = username
        4. password(string) = password
        5. logfile(string) = logfile name
        6. timeout(int) = timeout duration
        7. prompt(string) = destination prompt
        8. conn_options(string) = extra arguments that will be used when
                                  sending the ssh/telnet command
        9. custom_keystroke(string) = keystroke(to be given after initial timeout)
        10. escape(string) = true/false(to set TERM as dump)

    :Returns:
        1. session_object(pexpect session object)
        2. conn_string(pre and post login message)
    """
    print_warning("This method is obsolete and will be deprecated soon. Please"
                  " use 'connect_ssh' method of 'PexpectConnect' class "
                  "in 'warrior/Framework/ClassUtils/WNetwork/warrior_cli_class.py'")

    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    session_object, conn_string = wc_obj.connect_ssh(
        ip=ip, port=port, username=username, password=password, logfile=logfile, timeout=timeout,
        prompt=prompt, conn_options=conn_options, custom_keystroke=custom_keystroke, escape=escape)

    return session_object, conn_string


@mocked
def connect_telnet(ip, port="23", username="", password="",
                   logfile=None, timeout=60, prompt=".*(%|#|\$)",
                   conn_options="", custom_keystroke="", escape="", **kwargs):
    """
    Initiates Telnet connection via a specific port. Creates log file.

    :Arguments:
        1. ip = destination ip
        2. port(string) = telnet port
        3. username(string) = username
        4. password(string) = password
        5. logfile(string) = logfile name
        6. timeout(int) = timeout duration
        7. prompt(string) = destination prompt
        8. conn_options(string) = extra arguments that will be used when
                                  sending the ssh/telnet command
        9. custom_keystroke(string) = keystroke(to be given after initial timeout)
        10. escape(string) = true/false(to set TERM as dump)

    :Returns:
        1. session_object(pexpect session object)
        2. conn_string(pre and post login message)
    """
    print_warning("This method is obsolete and will be deprecated soon. Please"
                  " use 'connect_telnet' method of 'PexpectConnect' class "
                  "in 'warrior/Framework/ClassUtils/WNetwork/warrior_cli_class.py'")

    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    session_object, conn_string = wc_obj.connect_telnet(
        ip=ip, port=port, username=username, password=password, logfile=logfile, timeout=timeout,
        prompt=prompt, conn_options=conn_options, custom_keystroke=custom_keystroke, escape=escape)

    return session_object, conn_string


def disconnect_telnet(child):
    """Disconnects a telnet session """
    print_warning("This method is obsolete and will be deprecated soon. Please"
                  " use 'disconnect_telnet' method of 'PexpectConnect' class "
                  "in 'warrior/Framework/ClassUtils/WNetwork/warrior_cli_class.py'")

    if isinstance(child, WNetwork.warrior_cli_class.WarriorCli):
        child = child.disconnect_telnet()
    elif isinstance(child, pexpect.spawn):
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = child
        wc_obj.disconnect_telnet()

    return child


def disconnect(child):
    """
    - Disconnects warrior_cli_class session object(pexpect/paramiko)
    - Returns session object(same child)
    """
    print_warning("This method is obsolete and will be deprecated soon. Please"
                  " use 'disconnect' method of 'PexpectConnect' class "
                  "in 'warrior/Framework/ClassUtils/WNetwork/warrior_cli_class.py'")

    if isinstance(child, WNetwork.warrior_cli_class.WarriorCli):
        child = child.disconnect()
    elif isinstance(child, pexpect.spawn):
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = child
        wc_obj.disconnect()

    return child


@mocked
def send_command_and_get_response(sessionobj, prompt1, prompt2, command):
    """"Sends a command to a terminal expects a completion prompt
    If completion prompt was found, returns the response of the command """

    print_warning("This method is obsolete and will be deprecated soon. Please"
                  " use 'send_command' method of 'PexpectConnect' class "
                  "in 'warrior/Framework/ClassUtils/WNetwork/warrior_cli_class.py'")

    if isinstance(sessionobj, WNetwork.warrior_cli_class.WarriorCli):
        _, response = sessionobj.send_command(prompt1, prompt2, command)
    elif isinstance(sessionobj, pexpect.spawn):
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = sessionobj
        _, response = wc_obj.send_command(prompt1, prompt2, command)
    else:
        response = ""
        print_warning("Unable to send the command since the sessionobj is not "
                      "supported by warrior_cli_class, status will be marked as ERROR."
                      "Please use warrior_cli_class for session establishment.")

    return response


def smart_analyze(prompt, testdatafile=None):
    """
        retrieve the correspond smart testdata file for smart cmd
        from either Tools/connection or testcase testdata file
        :param prompt:
            The string that will be analyzed in order to find the device system
        :param testdatafile:
            optional arg to provide a pre-defined device system in the test datafile
        :return:
            the smart datafile that contains the smart cmd to be sent
    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    smart_datafile = wc_obj.smart_analyze(prompt, testdatafile)

    return smart_datafile


def send_smart_cmd(connect_testdata, session_object, tag_value, call_system_name, pre_tag):
    """
        The beacons of Gondor are lit
        send out the smart command
        :param connect_testdata:
            the smart testdata file that contains the smart cmd
        :param session_object:
            use this pexpect object to send out command
        :param tag_value:
            specify the testdata block of commands that get sent out
        :param call_system_name:
            in order to get passed the substitutions, a system name must be provided
        :param pre_tag:
            Distinguish if it is a connect smart action or disconnect smart action
    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    wc_obj.send_smart_cmd(connect_testdata, session_object, tag_value, call_system_name, pre_tag)

def smart_action(datafile, call_system_name, raw_prompt, session_object, tag_value,
                 connect_testdata=None):
    """
        entry function for sending smart command
        :param datafile:
            the testcase datafile
        :param call_system_name:
            in order to get passed the substitutions, a system name must be provided
        :param raw_prompt:
            The string that will be analyzed in order to find the device system
        :param session_object:
            use this pexpect object to send out command
        :param tag_value:
            specify the testdata block of commands that get sent out
        :param connect_testdata:
            the smart testdata file that contains the smart cmd, optional in here
        :return:
            the smart testdata file that contains the smart cmd
    """

    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    smart_testdatafile = wc_obj.smart_action(datafile, call_system_name, raw_prompt,
                                             session_object, tag_value, connect_testdata)
    return smart_testdatafile


def get_connection_port(conn_type, inpdict):
    """Gets the port for ssh or telnet connections
    1. ssh :
        - looks if ssh_port is present in  inpdict.
        - if not checks for conn_port
        - if both not present returns None
    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    inpdict = wc_obj.get_connection_port(conn_type, inpdict)
    return inpdict


@mocked
def send_command(session_object, start_prompt, end_prompt, command,
                 timeout=60):
    """
    Sends command to warrior_cli_class session object(pexpect/paramiko)
    and returns the status of the command sent.
    - Checks for the availability of the start_prompt.
    - if start prompt was available sends the command
    - if failure response is not None and failure response found in
    response then returns False.
    - else if failure response was not found
    and end prompt also not found returns False.
    - else if failure response was not found and end prompt found,
    then returns true.
    """
    print_warning("This method is obsolete and will be deprecated soon. Please"
                  " use 'send_command' method of 'PexpectConnect' class "
                  "in 'warrior/Framework/ClassUtils/WNetwork/warrior_cli_class.py'")

    if isinstance(session_object, WNetwork.warrior_cli_class.WarriorCli):
        status, response = session_object.send_command(start_prompt, end_prompt,
                                                       command, timeout)
    elif isinstance(session_object, pexpect.spawn):
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = session_object
        status, response = wc_obj.send_command(start_prompt, end_prompt, command, timeout)
    else:
        status, response = "ERROR", ""
        print_warning("Unable to send the command since the session_object is not an "
                      "instance of warrior_cli_class, status will be marked as ERROR. "
                      "Please use warrior_cli_class for session establishment.")

    return status, response


def sendPing(hostname, count, fname):
    """Sends a ping command
    :Arguments:
        1. count(string) = no of pings to be sent
        2. src_iface(string) = source interface from whihc ping messages
                                are to be sent.
        3. destip(string) = the destination ip address to ping.
        4. fname = logfile to log ping response.
    :Returns:
        status = boolean
    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    status = wc_obj.sendPing(hostname, count, fname)

    return status


def sendSourcePing(count, src_iface, destip, fname):
    """Sends a source based ping command
    i.e. if multiple interfaces are configured and available,
    sends pings from the src_iface provided
    :Arguments:
        1. count(string) = no of pings to be sent
        2. src_iface(string) = source interface from whihc ping messages
                                are to be sent.
        3. destip(string) = the destination ip address to ping.
        4. fname = logfile to log ping response.
    :Returns:
        status = boolean
    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    status = wc_obj.sendSourcePing(count, src_iface, destip, fname)

    return status


def send_commands_from_testdata(testdatafile, obj_session, **args):
    """
    - Parses the testdata file and gets the command details
    for rows marked execute=yes and row=str_rownum.
    - Sends the obtained commands to the warrior_cli_class session object(obj_Session).
    - If the commands have verification attribute set,
    then verifies the verification text for presence/absence as defined
    in the respective found attribute in the testdatfile.

    :Arguments:
        1. testdatafile = the xml file where command details are available
        2. obj_session = pexpect or warrior_cli_class session object
        3. logfile = logfile of the pexpect session object.
        4. varconfigfile=  xml file from which the values will be taken for subtitution
        5. var_sub(string) = the pattern [var_sub] in the testdata commands,
                                 start_prompt, end_prompt, verification search
                                 will substituted with this value.
        6. args = Optional filter to specify title/rownum
    :Returns:
        1. finalresult = boolean
    """
    if isinstance(obj_session, WNetwork.warrior_cli_class.WarriorCli):
        finalresult, td_resp_dict = obj_session.send_commands_from_testdata(testdatafile, **args)
    else:
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = obj_session
        finalresult, td_resp_dict = wc_obj.send_commands_from_testdata(testdatafile, **args)

    return finalresult, td_resp_dict


@mocked
def _send_cmd(obj_session, **kwargs):
    """method to send command based on the type of object """

    if isinstance(obj_session, WNetwork.warrior_cli_class.WarriorCli):
        result, response = obj_session._send_cmd(**kwargs)
    elif isinstance(obj_session, pexpect.spawn):
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = obj_session
        result, response = wc_obj._send_cmd(**kwargs)
    elif isinstance(obj_session, ssh_utils_class.SSHComm):
        command = kwargs.get('command')
        result, response = obj_session.get_response(command)
        print_info(response)
    else:
        print_warning('session object type is not supported')

    return result, response


def _get_response_dict(details_dict, index, response, response_dict):
    """Get the response dict for a command. """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    status, resp_key_list = \
        wc_obj._get_response_dict(details_dict, index, response, resp_key_list=[])
    return status, resp_key_list


def start_threads(started_thread_for_system, thread_instance_list, same_system,
                  unique_log_verify_list, system_name):
    """ This function iterates over unique_log_verify_list which consists of unique values
    gotten from monitor attributes and verify_on attributes

    If a system_name has a * against it, it indicates that the system is the
    same as the one on which the testcase is running. Thread would not be
    started for that system.

    :Returns:

    started_thread_for_system (list[str]) = Stores the system names for which
    threads were succesfully created

    thread_instance_list (list[str]) = stores the instances of thread created
    for corresponding system in the started_thread_for_system list,

    same_system (list[str]) = stores the system name which was the same as the
    system on which the TC is running without the trailing *,

    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    started_thread_for_system, thread_instance_list, same_system = \
        wc_obj.start_threads(started_thread_for_system, thread_instance_list,
                             same_system, unique_log_verify_list, system_name)

    return started_thread_for_system, thread_instance_list, same_system


def get_response_dict(started_thread_for_system, thread_instance_list,
                      same_system, response):
    """This function iterates over thread_instance_list and gets the data that
    the threads have stored in its data variable. Updates the remote_resp_dict
    with the system name and the corresponding data collected.

    The system names in same_system also get stored in the remote_resp_dict but
    their value is the same as the response that was obtained through the
    _send_cmd function

    :Returns:

    remote_resp_dict (dict) with collected logs as value to the system_name key
    """

    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    remote_resp_dict = wc_obj.get_response_dict(started_thread_for_system,
                                                thread_instance_list,
                                                same_system, response)

    return remote_resp_dict


def get_unique_log_and_verify_list(log_list, verify_on_list, system_name):
    """This function loops through the log_list and the verify_on_list
    and returns a unique list containing unique sustem names fromboth the lists
    """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    final_list = wc_obj.get_unique_log_and_verify_list(log_list, verify_on_list, system_name)

    return final_list


def _send_cmd_get_status(obj_session, details_dict, index, system_name=None):
    """Sends a command, verifies the response and returns
    status of the command """
    if isinstance(obj_session, WNetwork.warrior_cli_class.WarriorCli):
        result, response = obj_session._send_cmd_get_status(details_dict,
                                                            index, system_name)
    else:
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = obj_session
        result, response = wc_obj._send_cmd_get_status(details_dict,
                                                       index, system_name)

    return result, response


def _get_obj_session(details_dict, obj_session, kw_system_name, index):
    """If system name is provided in testdata file
    get the session of that system name and use it or
    use the current obj_session"""
    if isinstance(obj_session, WNetwork.warrior_cli_class.WarriorCli):
        value, kw_system_name, details_dict = obj_session._get_obj_session(
            details_dict, kw_system_name, index)
    else:
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = obj_session
        value, kw_system_name, details_dict = wc_obj._get_obj_session(
            details_dict, kw_system_name, index)

    return value, kw_system_name, details_dict

@mocked
def _send_command_retrials(obj_session, details_dict, index, **kwargs):
    """ Sends a command to a session, if a user provided pattern
    is found in the command response then tries to resend the command multiple
    times.
    retry_timer = time interval between subsequent retries
    retry_onmatch = the pattern to be matched in the response
                    in order to retry the command.
    retry_count = no of times to retry.
    """
    if isinstance(obj_session, WNetwork.warrior_cli_class.WarriorCli):
        result, response = obj_session._send_command_retrials(details_dict, index, **kwargs)
    else:
        wc_obj = WNetwork.warrior_cli_class.WarriorCli()
        wc_obj.conn_obj = WNetwork.warrior_cli_class.PexpectConnect()
        wc_obj.conn_obj.target_host = obj_session
        result, response = wc_obj._send_command_retrials(details_dict, index, **kwargs)

    return result, response


def _get_match_status(retry_onmatch, response):
    """ Searchs retry_onmatch value in response """
    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    status = wc_obj._get_match_status(retry_onmatch, response)
    return status


def _send_cmd_by_type(session_object, command):
    """Determine the command type and send accordingly
    """

    wc_obj = WNetwork.warrior_cli_class.WarriorCli()
    wc_obj._send_cmd_by_type(session_object, command)

##################
# Uses telnet_Utils.Tnet_Comm object as input
# def send_telnet_command(telnet_session, command, start_prompt='',
#                         end_prompt='', failure_resp=''):
#     """ Takes a telnet session object created using
#     telnet_Utils.Tnet_Comm object as input
#     Sends a tl1 command to the telnet session and verifies the end
#     prompt/failure response
#     :Returns:
#         1. True, response = if end prompt is encountered
#         2. False, response = if failure_resp is encountered
#     """
#     #print "start prompt is : {0}".format(start_prompt)
#     msg = 'Send Command:{0}'.format(command)
#     pNote(msg)
#     try:
#         response = telnet_session.get_response(command, end_prompt)
#         print_info("Response:{0}".format(response))
#     except Exception as exception:
#         print_debug("Could not find the end prompt %s or failure "
#                     "response %s "% (end_prompt, failure_resp))
#         err = print_exception(exception)
#         return False, str(err)
#
#     e_prompt = re.compile(end_prompt, re.DOTALL)
#     f_resp = re.compile(failure_resp, re.DOTALL)
#
#
#     if re.search(f_resp, response):
#         return (False, response)
#     elif re.search(e_prompt, response):
#         return (True, response)
#     else:
#         return (False, response)



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

import os
import re
import sys
import time
import subprocess
import getpass
import xml.etree.ElementTree as ET
import ast
from distutils.version import LooseVersion
from warrior.Framework import Utils
from warrior.Framework.Utils.print_Utils import print_info, print_debug,\
 print_warning, print_exception, print_error, print_without_logging
from warrior.Framework.Utils.testcase_Utils import pNote
from warrior.Framework.ClassUtils import database_utils_class
from warriornetwork.ClassUtils.WNetwork.loging import ThreadedLog
from warriornetwork.Utils.list_Utils import get_list_by_separating_strings
from warrior.Framework.Utils.data_Utils import get_object_from_datarepository
from warrior.WarriorCore.Classes.warmock_class import mocked
from timeit import itertools
from warrior.Framework.Utils.config_Utils import data_repository
from warrior.WarriorCore.Classes.war_cli_class import WarriorCliClass
from multiprocessing import current_process

""" Module for performing CLI operations """

class WarriorCli(object):
    """
    Class to handle CLI operations.
    """

    def __init__(self):
        """ Constructor """
        self.conn_obj = None

    def connect_ssh(self, ip, port="22", username="", password="",
                    logfile=None, timeout=60, prompt=".*(%|#|\$)",
                    conn_options="", custom_keystroke="", escape=""):
        """
        - Initiates SSH connection via a specific port. Creates log file.
        :Arguments:
            1. ip = destination ip
            2. port(string) = telnet port
            3. username(string) = username
            4. password(string) = password
            5. logfile(string) = logfile name
            6. timeout(int) = timeout duration
            7. prompt(string) = destination prompt
            8. conn_options(string) = extra arguments that will be used when
                                      sending the ssh/telnet command
            9. custom_keystroke(string) = keystroke(to be given after initial
                                          timeout)
            10. escape(string) = true/false(to set TERM as dump)
        :Returns:
            1. session_object(pexpect session object)
            2. conn_string(pre and post login message)
        """

        credentials = {}
        credentials['ip'] = ip
        credentials['port'] = port
        credentials['username'] = username
        credentials['password'] = password
        credentials['logfile'] = logfile
        credentials['timeout'] = timeout
        credentials['prompt'] = prompt
        credentials['conn_options'] = conn_options
        credentials['custom_keystroke'] = custom_keystroke
        credentials['escape'] = escape

        self.conn_obj = PexpectConnect(credentials)
        self.conn_obj.connect_ssh()

        return self.conn_obj.target_host, self.conn_obj.conn_string

    def connect_telnet(self, ip, port="23", username="", password="",
                       logfile=None, timeout=60, prompt=".*(%|#|\$)",
                       conn_options="", custom_keystroke="", escape=""):
        """
        Initiates Telnet connection via a specific port. Creates log file.
        :Arguments:
            1. ip = destination ip
            2. port(string) = telnet port
            3. username(string) = username
            4. password(string) = password
            5. logfile(string) = logfile name
            6. timeout(int) = timeout duration
            7. prompt(string) = destination prompt
            8. conn_options(string) = extra arguments that will be used when
                                      sending the ssh/telnet command
            9. custom_keystroke(string) = keystroke(to be given after initial
                                          timeout)
            10. escape(string) = true/false(to set TERM as dump)
        :Returns:
            1. session_object(pexpect session object)
            2. conn_string(pre and post login message)
        """

        credentials = {}
        credentials['ip'] = ip
        credentials['port'] = port
        credentials['username'] = username
        credentials['password'] = password
        credentials['logfile'] = logfile
        credentials['timeout'] = timeout
        credentials['prompt'] = prompt
        credentials['conn_options'] = conn_options
        credentials['custom_keystroke'] = custom_keystroke
        credentials['escape'] = escape

        self.conn_obj = PexpectConnect(credentials)
        self.conn_obj.connect_telnet()

        return self.conn_obj.target_host, self.conn_obj.conn_string

    def disconnect(self):
        """
        Disconnects pexpect/paramiko session
        """

        if self.conn_obj and self.conn_obj.target_host:
            self.conn_obj.disconnect()

    def disconnect_telnet(self):
        """
        Disconnects pexpect telnet session
        """

        if self.conn_obj and self.conn_obj.target_host:
            self.conn_obj.disconnect_telnet()

    @mocked
    def send_command(self, start_prompt, end_prompt, command,
                     timeout=60):
        """ Sends the command to ssh/telnet session
        :Arguments:
            1. start_prompt(string) = expected start prompt
            2. end_prompt(string) = expected end prompt
            3. command(string) = command to be executed
            4. timeout(int) = timeout to wait for the end prompt
            5. child = pexpect/paramiko object
            6. childType = pexpect/paramiko
        :Returns:
            1. status(boolean)
            2. response - command execution response
        """

        status = False
        response = ""

        if self.conn_obj and self.conn_obj.target_host:
            status, response = self.conn_obj.send_command(
                command=command, start_prompt=start_prompt,
                end_prompt=end_prompt, timeout=timeout)

        return status, response

    def send_commands_from_testdata(self, testdatafile, **args):
        """
        - Parses the testdata file and gets the command details
        for rows marked execute=yes and row=str_rownum.
        - Sends the obtained commands to the warrior_cli_class
          session object(obj_Session).
        - If the commands have verification attribute set,
        then verifies the verification text for presence/absence as defined
        in the respective found attribute in the testdatfile.

        :Arguments:
            1. testdatafile = the xml file where command details are available
            2. logfile = logfile of the pexpect session object.
            3. varconfigfile = xml file from which the values will be taken
                               for substitution
            4. var_sub(string) = the pattern [var_sub] in the testdata
                                 commands, start_prompt, end_prompt,
                                 verification search will substituted
                                 with this value.
            5. args = Optional filter to specify title/rownum
        :Returns:
            1. finalresult = boolean
        """
        responses_dict = {}
        resp_key_list = []
        td_resp_dict = {}
        varconfigfile = args.get('varconfigfile', None)
        datafile = args.get("datafile", None)
        var_sub = args.get('var_sub', None)
        title = args.get('title', None)
        row = args.get('row', None)
        system_name = args.get("system_name")
        session_name = args.get("session_name")
        if session_name is not None:
            system_name = system_name + "." + session_name
        testdata_dict = Utils.data_Utils.get_command_details_from_testdata(
            testdatafile, varconfigfile, var_sub=var_sub, title=title, row=row,
            system_name=system_name, datafile=datafile)
        finalresult = True if len(testdata_dict) > 0 else False
        for key, details_dict in testdata_dict.items():
            responses_dict[key] = ""
            command_list = details_dict["command_list"]
            stepdesc = "Send the following commands: "
            pNote(stepdesc)
            n = 0
            for commands in command_list:
                if details_dict["logmsg_list"][n] and details_dict["logmsg_list"][n].lower() == "false":
                    commands = "**************"
                pNote("Command #{0}\t: {1}".format((n+1), commands))
                n = n + 1
            intsize = len(command_list)
            if intsize == 0:
                finalresult = False

            # Send Commands
            for i in range(0, intsize):
                print_info("")
                print_debug(">>>")
                command = details_dict["command_list"][i]
                if not details_dict["logmsg_list"][i] or details_dict["logmsg_list"][i].lower() != "false":
                    pNote("Command #{0}\t: {1}".format(str(i+1), command))
                new_obj_session, td_sys, details_dict, session_id = \
                    self._get_obj_session(details_dict, system_name, index=i, sess_id=True)
                if new_obj_session:
                    result, response = new_obj_session._send_cmd_get_status(
                        details_dict, index=i, system_name=td_sys)
                    result, response = new_obj_session._send_command_retrials(
                        details_dict, index=i, result=result,
                        response=response, system_name=td_sys)
                    rspRes, resp_key_list = new_obj_session._get_response_dict(
                        details_dict, i, response, resp_key_list)
                    if len(resp_key_list) != i+1:
                        resp_key_list.append({})
                    resp_session_id = session_id + "_td_response"

                    td_resp_dict = self.update_resp_ref_to_repo(details_dict, resp_key_list, i,
                                                                key, td_resp_dict, resp_session_id,
                                                                result)

                    result = (result and rspRes) if "ERROR" not in (result, rspRes) else "ERROR"
                    print_debug("<<<")
                else:
                    finalresult = "ERROR"
                    pNote("COMMAND STATUS:{0}".format(finalresult))
                    print_debug("<<<")
                    continue

                if result == "ERROR" or finalresult == "ERROR":
                    result = "ERROR"
                    finalresult = "ERROR"
                finalresult = finalresult and result
                if details_dict["return_on_fail_list"][i] and (result == "ERROR" or result is False):
                    if details_dict["return_on_fail_list"][i].lower() == "yes":
                        pNote("skipping remaining commands, if exists")
                        break;
            responses_dict[key] = {k: v for d in resp_key_list for k, v in d.items()}
        return finalresult, td_resp_dict

    def update_resp_ref_to_repo(self, details_dict, resp_key_list, i,
                                title_row, td_resp_dict, session_id, status=True):
        """
        Updates the response reference in appropriate session_id.
        There are two cases:
            1. The user gives the system and session name only in testcase,
            then it updates on testcase's session id
            2. The user gives the system and session name in both testcase and testdata file,
            then it takes testdata as priority and updates on testdata's session id
        """
        step_title_row = 'step_{}_{}'.format(data_repository['step_num'], title_row)
        try:
            for resp in list(resp_key_list[i].keys()):
                td_resp_dict = get_object_from_datarepository(str(session_id))
                # checks if title_row value is not in td_resp_dict
                loop_id = get_object_from_datarepository("loopid")
                iter_number = get_object_from_datarepository("loop_iter_number")
                if loop_id is not None and iter_number is not None:
                    title_row = title_row + '_' + str(loop_id) + '_' + str(iter_number)
                    step_title_row = step_title_row + '_' + str(loop_id) + '_' + str(iter_number)
                if title_row not in td_resp_dict:
                    # if not available then it first updates the
                    # title_row value to td_resp_dict
                    td_resp_dict[title_row] = {}
                if step_title_row not in td_resp_dict:
                    # if not available then it first updates the
                    # step_title_row value to td_resp_dict
                    td_resp_dict[step_title_row] = {}
                # updates td_resp_dict with the key and value
                status = {True: "PASS", False: "FAIL", "ERROR": "ERROR"}.get(status)
                temp_resp = {resp: resp_key_list[i][resp], resp+"_status": status,
                             resp+"_command": details_dict["command_list"][i]}
                td_resp_dict[title_row].update(temp_resp)
                td_resp_dict[step_title_row].update(temp_resp)
                if not details_dict["logmsg_list"][i] or details_dict["logmsg_list"][i].lower() != "false":
                    print_debug("Portion of response saved to the data "
                          "repository with key: '{0}.{1}.{2}' and value: '{3}'"
                          .format(session_id, title_row, resp, temp_resp[resp]))
        except Exception as e:
            print_error("Found exception: {}".format(e))
            print_error("Check if the key list below is not empty and the keys are proper")
            print_error(resp_key_list)
        return td_resp_dict

    @mocked
    def _send_cmd(self, **kwargs):
        """method to send command based on the type of object """

        result = False
        response = ""
        if self.conn_obj and self.conn_obj.target_host:
            command = kwargs.get('command')
            startprompt = kwargs.get('startprompt', ".*")
            endprompt = kwargs.get('endprompt', None)
            cmd_timeout = kwargs.get('cmd_timeout', None)
            log = kwargs.get('log', "true")
            sleep_before = kwargs.get('sleep_before', None)
            result, response = self.conn_obj.send_command(command, startprompt,
                                                          endprompt,
                                                          cmd_timeout,
                                                          sleep_before=sleep_before,
                                                          log=log)
        return result, response

    @staticmethod
    def _get_response_dict(details_dict, index, response, resp_key_list):
        """Get the response dict for a command. """
        def print_warn_msg(keyvars, numpats):
            """ print a warning message if the number of vars to be stored with
            the patterns does not match with the number of patterns
            """
            warn_msg = ("The number of response reference keys to store is {0}"
                        " than \nthe response reference patterns.\nThe number "
                        "of response reference keys({1}) is {2}\nwhich is {0} "
                        "than number of response reference patterns {3}")
            lessormore = "less" if len(keyvars) < numpats else "more"
            print_warning(warn_msg.format(lessormore, ", ".join(keyvars),
                                          len(keyvars), numpats))
        resp_ref = details_dict["resp_ref_list"][index]
        resp_req = details_dict["resp_req_list"][index]
        resp_pat_req = details_dict["resp_pat_req_list"][index]
        resp_keys = details_dict["resp_key_list"][index]
        inorder = details_dict["inorder_resp_ref_list"][index]
        resp_pat_key = details_dict["resp_pat_key_list"][index]
        status = True
        if inorder is not None and inorder.lower().startswith("n"):
            inorder = False
        else:
            inorder = True

        resp_req = {None: 'y', '': 'y',
                    'no': 'n', 'n': 'n'}.get(str(resp_req).lower(), 'y')
        resp_ref = {None: str(index+1), '': str(index+1)}.get(resp_ref, str(resp_ref))
        resp_pat_key = {None: str(index + 1), '': str(index + 1)}.get(resp_pat_key, str(resp_pat_key))
        if not resp_req == "n":
            save_msg1 = "User has requested saving response"
            save_msg2 = "Response pattern required by user is : {0}"
            save_msg4 = "Cannot found response pattern: {0} in response"
            if resp_pat_req is not None:
                # if the requested pattern not found return empty string
                reobj = re.search(resp_pat_req, response)
                if reobj is None:
                    pNote(save_msg4.format(resp_pat_req))
                response = reobj.group(0) if reobj is not None else ""
                temp_resp_dict = {resp_ref: response}
                resp_key_list.append(temp_resp_dict)
                # storing the value and expected output in data repository 
                resp_key_dict = {resp_pat_key: response}
                data_repository.update(resp_key_dict)
                print_debug(save_msg1+'.')
                print_debug(save_msg2.format(resp_pat_req))
            elif resp_keys is not None:
                keys = resp_ref.split(',')
                resp_pat_list = resp_pat_key.split(',')
                # get the patterns from pattern entries in testdata file
                patterns = []
                # error out if any of the key element can not be retrieved
                for key, resp_key in itertools.zip_longest(keys, resp_keys):
                    if isinstance(resp_key, ET.Element):
                        patterns.append(resp_key.get("resp_pattern_req"))
                    elif isinstance(resp_key, str):
                        print_error("There is no pattern element for key '{}' corresponding"
                                    " to '{}', please check".format(resp_key, key))
                        status = False
                else:
                    if status is False:
                        return status, resp_key_list
                # warn if number of patterns does not match number of resp_ref keys
                if len(keys) != len(patterns):
                    print_warn_msg(keys, len(patterns))
                if inorder:
                    pNote(save_msg1+' inorder.')
                    # since inorder pattern matching selected, join all the
                    # patterns in order to create a single big pattern
                    cpatterns = ["({})".format(pat) for pat in patterns]
                    pattern = ".*".join(cpatterns)
                    if pattern.endswith(".*(.*)"):
                        # remove .* pattern from above
                        pattern = pattern[:-6]+pattern[-4:]
                    reobj = re.search(pattern, response, re.DOTALL)
                    if reobj:
                        grps = reobj.groups()
                        # update resp_key_list with resp_ref keys and
                        # their corresponding matched patterns
                        resp_key_list.append(dict(list(zip(keys, grps))))
                        # To process multiple pattern values from testdata
                        for resp, grps in zip(patterns, resp_pat_list):
                            resp_key_dict = {grps: resp}
                            data_repository.update(resp_key_dict)
                        pNote(save_msg2.format(pattern))
                    else:
                        print_error("inorder search of patterns in response "
                                    "failed")
                        print_error("Expected: '{}'".format(pattern))
                        print_error("But Found: '{}'".format(response))
                        status = False
                else:
                    temp_resp_key_list = []
                    pNote(save_msg1+' separately.')
                    for key, pattern in zip(keys, patterns):
                        reobj = re.search(pattern, response)
                        presponse = reobj.group(0) if reobj is not None else ""
                        temp_resp_key_list.append(presponse)
                        pNote(save_msg2.format(pattern))
                        # To process multiple values from testdata
                        for resp, pattern in zip(resp_pat_list, patterns):
                            reobj = re.search(pattern, response)
                            presponse = reobj.group(0) if reobj is not None else ""
                            resp_key_dict = {resp: presponse}
                            data_repository.update(resp_key_dict)
                    resp_key_list.append(dict(list(zip(keys, temp_resp_key_list))))
            else:
                temp_resp_dict = {resp_ref: response}
                resp_key_list.append(temp_resp_dict)
        else:
            temp_resp_dict = {resp_ref: ""}
            resp_key_list.append(temp_resp_dict)
        return status, resp_key_list

    @staticmethod
    def start_threads(started_thread_for_system, thread_instance_list,
                      same_system, unique_log_verify_list, system_name):
        """This function iterates over unique_log_verify_list which consists of
         unique values gotten from monitor attributes and verify_on attributes

        If a system_name has a * against it, it indicates that the system is
        the same as the one on which the testcase is running. Thread would not
        be started for that system.

        :Returns:
        started_thread_for_system (list[str]) = Stores the system names for
        which threads were succesfully created

        thread_instance_list (list[str]) = stores the instances of thread
        created for corresponding system in the started_thread_for_system list,

        same_system (list[str]) = stores the system name which was the same as
        the system on which the TC is running without the trailing *,
        """
        started_thread_for_system = []
        thread_instance_list = []
        same_system = []
        for i in range(0, len(unique_log_verify_list)):
            if unique_log_verify_list[i] == system_name:
                temp_list = unique_log_verify_list[i].split(".")
                if len(temp_list) > 1:
                    unique_log_verify_list[i] = \
                     Utils.data_Utils.get_session_id(temp_list[0],
                                                     temp_list[1])
                else:
                    unique_log_verify_list[i] = \
                     Utils.data_Utils.get_session_id(temp_list[0])
                same_system.append(unique_log_verify_list[i])
            else:
                if unique_log_verify_list[i]:
                    temp_list = unique_log_verify_list[i].split(".")
                    if len(temp_list) > 1:
                        unique_log_verify_list[i] = \
                         Utils.data_Utils.get_session_id(temp_list[0],
                                                         temp_list[1])
                    else:
                        unique_log_verify_list[i] = \
                         Utils.data_Utils.get_session_id(temp_list[0])
                    datarep_obj = \
                        Utils.data_Utils.get_object_from_datarepository(
                            unique_log_verify_list[i])
                    if datarep_obj is False:
                        print_info("{0} does not exist in data repository"
                                   .format(unique_log_verify_list[i]))
                    else:
                        try:
                            new_thread = ThreadedLog()
                            new_thread.start_thread(datarep_obj)
                            print_info("Collecting response from: "
                                       "{0}".format(unique_log_verify_list[i]))
                            started_thread_for_system.append(
                                unique_log_verify_list[i])
                            thread_instance_list.append(new_thread)
                        except:
                            print_info("Unable to collect response from: "
                                       "{0}".format(unique_log_verify_list[i]))
        return started_thread_for_system, thread_instance_list, same_system

    @staticmethod
    def get_response_dict(started_thread_for_system, thread_instance_list,
                          same_system, response):
        """
        This function iterates over thread_instance_list and gets the data that
        the threads have stored in its data variable. Updates remote_resp_dict
        with the system name and the corresponding data collected.

        The system names in same_system also get stored in the remote_resp_dict
        but their value is the same as the response that was obtained through
        the _send_cmd function

        :Returns:

        remote_resp_dict (dict) with collected logs as value to the
        system_name key
        """
        remote_resp_dict = {}
        for i in range(0, len(same_system)):
            remote_resp_dict[same_system[i]] = response

        for i in range(0, len(started_thread_for_system)):
            data = thread_instance_list[i].data
            thread_instance_list[i].stop_thread()
            pNote("\n\n++++++++++++++++++++++++ RESPONSE FROM SYSTEM: {0} "
                  "++++++++++++++++++++\n\n".format(
                      started_thread_for_system[i]))
            pNote(data)
            pNote("\n\n++++++++++++++++++++++++ END OF DATA FROM SYSTEM: {0} "
                  "++++++++++++++++++++\n\n".format(
                      started_thread_for_system[i]))
            remote_resp_dict[started_thread_for_system[i]] = data

        if len(started_thread_for_system) > 0:
            print_info("Waiting for maximum of 30 seconds to stop collecting "
                       "logs from verify_on system(s)")

        for i in range(0, len(started_thread_for_system)):
            thread_instance_list[i].join_thread(timeout=30, retry=3)
            if thread_instance_list[i].thread_status() is True:
                print_error("Unable to stop collecting logs from {0}."
                            "Please check below message for all "
                            "exception trace that occurred: "
                            "\n{1}".format(started_thread_for_system[i],
                                           thread_instance_list[i].
                                           stop_thread_err_msg))
        return remote_resp_dict

    @staticmethod
    def get_unique_log_and_verify_list(log_list, verify_on_list, system_name):
        """This function loops through the log_list and the verify_on_list and
        returns a unique list containing unique sustem names fromboth the lists
        """
        final_list = []
        if log_list is not None and log_list != "" and log_list is not False:
            comma_sep_log_names = log_list.split(",")
            for i in range(0, len(comma_sep_log_names)):
                comma_sep_log_names[i] = comma_sep_log_names[i].strip()
        else:
            comma_sep_log_names = []

        comma_sep_verify_names = []
        if verify_on_list is not None and verify_on_list != "" and \
           verify_on_list is not False:
            for i in range(0, len(verify_on_list)):
                if verify_on_list[i] is not None and verify_on_list[i] != "" \
                 and verify_on_list[i] is not False:
                    temp_list = verify_on_list[i].split(",")
                    for j in range(0, len(temp_list)):
                        comma_sep_verify_names.append(temp_list[j].strip())
                else:
                    comma_sep_verify_names.append([])
        else:
            comma_sep_verify_names = []

        for i in range(0, len(comma_sep_log_names)):
            if comma_sep_log_names[i] not in final_list:
                final_list.append(comma_sep_log_names[i])

        for i in range(0, len(comma_sep_verify_names)):
            if comma_sep_verify_names[i] == "":
                comma_sep_verify_names[i] = system_name
            if comma_sep_verify_names[i] not in final_list:
                final_list.append(comma_sep_verify_names[i])
        return final_list

    def _send_cmd_get_status(self, details_dict, index, system_name=None):
        """Sends a command, verifies the response and returns
        status of the command """
        command = details_dict["command_list"][index]
        startprompt = details_dict["startprompt_list"][index]
        endprompt = details_dict["endprompt_list"][index]
        verify_list = details_dict["verify_list"][index]
        cmd_timeout = details_dict["timeout_list"][index]
        verify_text_list = details_dict["verify_text_list"][index]
        verify_context_list = details_dict["verify_context_list"][index]
        sleeptime = details_dict["sleeptime_list"][index]
        resp_ref = details_dict["resp_ref_list"][index]
        resp_req = details_dict["resp_req_list"][index]
        resp_pat_req = details_dict["resp_pat_req_list"][index]
        resp_pat_key = details_dict["resp_pat_key_list"][index]
        verify_on_list = details_dict["verify_on_list"][index]
        log_list = details_dict["log_list"][index]
        inorder_search = details_dict["inorder_search_list"][index]
        varconfigfile = details_dict["vc_file_list"][index]
        operator = details_dict["operator_list"][index]
        cond_value = details_dict["cond_value_list"][index]
        cond_type = details_dict["cond_type_list"][index]
        sleeptime_before_match = details_dict["sleeptime_before_match_list"][index]
        unique_log_verify_list = self.get_unique_log_and_verify_list(
            log_list, verify_on_list, system_name)
        log = details_dict["logmsg_list"][index] or "true"

        startprompt = {None: ".*", "": ".*"}.get(startprompt, str(startprompt))
        resp_req = {None: 'y', '': 'y',
                    'no': 'n', 'n': 'n'}.get(str(resp_req).lower(), 'y')
        resp_ref = {None: index+1, '': index+1}.get(resp_ref, str(resp_ref))
        resp_pat_req = {None: ""}.get(resp_pat_req, str(resp_pat_req))
        resp_pat_key = {None: index + 1, '': index + 1}.get(resp_pat_key, str(resp_pat_key))
        sleeptime = {None: 0, "": 0, "none": 0, False: 0, "false": 0}.get(
            str(sleeptime).lower(), str(sleeptime))
        sleeptime = int(sleeptime)

        if inorder_search is not None and \
           inorder_search.lower().startswith("y"):
            inorder_search = True
        else:
            inorder_search = False
        if log is None or log.lower() != "false":
            pNote("System name\t: {0}".format(system_name))
            pNote("Startprompt\t: {0}".format(startprompt))
            pNote("Endprompt\t: {0}".format(endprompt))
            pNote("Sleeptime\t: {0}".format(sleeptime))
            pNote("Response required: {0}".format(resp_req))
            pNote("Response reference: {0}".format(resp_ref))
            pNote("Response pattern required: {0}".format(resp_pat_req))
            pNote("Response pattern key: {0}".format(resp_pat_key))
        if not command:
            pNote("Received a boolean False or None type instead of a string "
                  "command, Command not provided or Variable substitution for "
                  "the command could have gone wrong", "error")
            pNote("Skipping execution of this command, result will be marked "
                  "as error", "debug")
            result = 'ERROR'
            response = ''
        else:
            started_thread_for_system, thread_instance_list, same_system = \
                self.start_threads([], [], [], unique_log_verify_list,
                                   system_name)

            result, response = self._send_cmd(startprompt=startprompt,
                                              endprompt=endprompt,
                                              command=command,
                                              cmd_timeout=cmd_timeout,
                                              sleep_before=sleeptime_before_match,
                                              log=log)

        if sleeptime > 0:
            if not WarriorCliClass.mock:
                pNote("Sleep time of '{0} seconds' requested post command "
                  "execution".format(sleeptime))
                Utils.datetime_utils.wait_for_timeout(sleeptime)
            else:
                pNote("Sleep time is skipped because as it is running in mock mode")

        try:
            remote_resp_dict = self.get_response_dict(
                started_thread_for_system, thread_instance_list, same_system,
                response)
        except NameError:
            remote_resp_dict = self.get_response_dict([], [], [], response)

        verify_on_list_as_list = get_list_by_separating_strings(
            verify_on_list, ",", system_name)
        if result and result is not 'ERROR':
            if verify_text_list is not None and verify_list is not None:
                verify_group = (operator, cond_value, cond_type)
                if inorder_search is True and len(verify_text_list) > 1:
                    result = Utils.data_Utils.verify_resp_inorder(
                        verify_text_list, verify_context_list, command,
                        response, varconfigfile, verify_on_list_as_list,
                        verify_list, remote_resp_dict, verify_group)
                else:
                    result = Utils.data_Utils.verify_resp_across_sys(
                        verify_text_list, verify_context_list, command,
                        response, varconfigfile, verify_on_list_as_list,
                        verify_list, remote_resp_dict, endprompt, verify_group, log)
        command_status = {True: "PASS", False: "FAIL", "ERROR": "ERROR"}.get(
            result)
        duration = data_repository.get("command_duration", None)
        if command_status == "PASS":
            pNote("COMMAND STATUS:{} | Duration: {} sec".format(command_status,
                                                                 duration))
        else:
            pNote("COMMAND STATUS:{0}".format(command_status))

        return result, response

    def _get_obj_session(self, details_dict, kw_system_name, index, sess_id=False):
        """If system name is provided in testdata file
        get the session of that system name and use it or
        use the current obj_session"""

        value = False
        kw_system_nameonly, _ = Utils.data_Utils.split_system_subsystem(
            kw_system_name)
        td_sys = details_dict["sys_list"][index]
        # To get the session name if it is provided as part of sys tag in td
        td_sys_split = td_sys.split('.') if isinstance(td_sys, str) else []
        if len(td_sys_split) == 2:
            td_sys = td_sys_split[0]
            session = td_sys_split[1]
        else:
            session = details_dict["session_list"][index]

        td_sys = td_sys.strip() if isinstance(td_sys, str) else td_sys
        td_sys = {None: False, False: False, "": False}.get(td_sys, td_sys)
        session = session.strip() if isinstance(session, str) else session
        session = {None: None, False: None, "": None}.get(session, session)
        if td_sys:
            system_name = kw_system_nameonly + td_sys if \
             td_sys.startswith("[") and td_sys.endswith("]") else td_sys
            session_id = Utils.data_Utils.get_session_id(system_name, session)
            obj_session = Utils.data_Utils.get_object_from_datarepository(
                session_id)
            if not obj_session:
                pNote("Could not find a valid connection for system_name={}, "
                      "session_name={}".format(system_name, session), "error")
                value = False
            else:
                value = obj_session
        else:
            # print obj_session
            value = self
            tc_sys_split = kw_system_name.split('.') if isinstance(kw_system_name, str) else []
            if len(tc_sys_split) == 2:
                system_name = tc_sys_split[0]
                session = tc_sys_split[1]
            else:
                system_name = kw_system_name
            session_id = Utils.data_Utils.get_session_id(system_name, session)

        if details_dict["sys_list"][index] is not None:
            kw_system_name = details_dict["sys_list"][index]
        if sess_id is True:
            return value, kw_system_name, details_dict, session_id

        return value, kw_system_name, details_dict

    def _send_command_retrials(self, details_dict, index, **kwargs):
        """ Sends a command to a session, if a user provided pattern
        is found in the command response then tries to resend the command
        multiple times.
        retry_timer = time interval between subsequent retries
        retry_onmatch = the pattern to be matched in the response
                        in order to retry the command.
        retry_count = no of times to retry.
        """
        retry = details_dict["retry_list"][index]
        retry = {None: 'n', '': 'n', 'none': 'n'}.get(str(retry).lower(),
                                                      retry)
        result = kwargs.get('result')
        response = kwargs.get('response')
        if retry == 'y' and (result is False or result == 'ERROR'):
            retry_timer = details_dict["retry_timer_list"][index]
            retry_onmatch = details_dict["retry_onmatch_list"][index]
            retry_count = details_dict["retry_count_list"][index]
            retry_timer = {None: 60, "": 60, "none": 60}.get(
                str(retry_timer).lower(), retry_timer)
            retry_count = {None: 5, "": 5, "none": 5}.get(
                str(retry_count).lower(), retry_count)
            print_info("")
            pNote("Retry was requested for the command")
            pNote("Command re-trials will begin since the most recent "
                  "command status was FAIL or ERROR")
            pNote("Retry count\t: {0}".format(retry_count))
            pNote("Retry timer\t: {0}".format(retry_timer))
            retry_onmatch = {None: False, "": False}.get(retry_onmatch,
                                                         str(retry_onmatch))
            print_onmatch = {False: ""}.get(retry_onmatch, str(retry_onmatch))
            pNote("Retry onmatch: {0}".format(print_onmatch))
            count = 0
            while count < int(retry_count):
                if result is False or result == 'ERROR':
                    match_status = self._get_match_status(retry_onmatch,
                                                          response)
                    if match_status:
                        count = count + 1
                        print_info("")
                        pNote("RETRIAL ATTEMPT:{0}".format(count))
                        pNote("Wait for {0}sec (retry_timer) before sending"
                              " the command again".format(retry_timer))
                        time.sleep(int(retry_timer))
                        result, response = self._send_cmd_get_status(
                            details_dict, index,
                            system_name=kwargs.get("system_name"))
                        command_status = {True: "PASS", False: "FAIL",
                                          "ERROR": "ERROR"}.get(result)
                        pNote("RETRIAL ATTEMPT:{0} STATUS:{1}".format(
                            count, command_status))
                    else:
                        break
                elif result is True:
                    break
        return result, response

    @staticmethod
    def _get_match_status(retry_onmatch, response):
        """ Searchs retry_onmatch value in response """
        status = True
        if retry_onmatch:
            pNote("Command will be executed again if "
                  "the pattern {0} is present in the "
                  "response of the previous execution of the command"
                  .format(retry_onmatch))
            match_object = re.search(retry_onmatch, response)
            if match_object:
                pNote("Found the pattern '{0}' "
                      "in the response of the previous execution "
                      "of the command".format(retry_onmatch))
            else:
                pNote("Did not find the pattern '{0}' "
                      "in the response of the previous execution "
                      "of the command".format(retry_onmatch))
                status = False
        return status

    def isalive(self):
        """
        Returns whether the paramiko/pexpect session is alive or not
        :Returns:
            True if alive else False
        """

        if self.conn_obj:
            target_host = self.conn_obj.target_host
        else:
            target_host = False

        if target_host:
            # for paramiko
            if self.conn_obj.conn_type == "SSH_NESTED":
                if target_host.get_transport():
                    status = target_host.get_transport().is_active()
                else:
                    status = False
            # for pexpect
            else:
                status = target_host.isalive()
        else:
            status = False

        return status

    def read_nonblocking(self, size=1024, timeout=None, *args, **kwargs):
        """
        Reads characters(size) from pexpect/paramiko session
        :Arguments:
            1. size = maximum number of bytes to read
        :Returns:
            1. read_string = Characters read from the pexpect/paramiko session
        """

        try:
            if self.conn_obj and self.conn_obj.target_host:
                if self.conn_obj.conn_type == "SSH_NESTED":
                    read_string = self.conn_obj.channel.recv(size)
                else:
                    read_string = self.conn_obj.target_host.\
                     read_nonblocking(size, timeout)
            else:
                read_string = ""
        except Exception:
            read_string = ""

        return read_string

    @property
    def timeout(self):
        """ Get session timeout(mins) value of pexpect/paramiko session  """

        timeout = None
        if self.conn_obj and self.conn_obj.target_host:
            if self.conn_obj.conn_type in ["SSH", "TELNET"]:
                timeout = self.conn_obj.target_host.timeout
            # elif self.conn_obj.conn_type == "SSH_NESTED":
            #    timeout = self.conn_obj.timeout
            #    # paramiko timeout value will be in seconds,
            #    # convert it to mins
            #    if timeout:
            #        timeout = timeout/60

        return timeout

    @timeout.setter
    def timeout(self, value):
        """ Set session timeout value(mins) for pexpect/paramiko session """

        if self.conn_obj and self.conn_obj.target_host:
            if self.conn_obj.conn_type in ["SSH", "TELNET"]:
                self.conn_obj.target_host.timeout = value
                # elif self.conn_obj.conn_type == "SSH_NESTED":
                #    # paramiko accepts timeout value in seconds
                #    self.conn_obj.timeout = value * 60

    @staticmethod
    def pexpect_spawn_with_env(pexpect_obj, command, timeout, escape=False,
                               env=None, pty_dimensions=None):
        """ spawn a pexpect object with environment & pty_dimensions variables """

        if env is None:
            env = {}

        kwargs = {'timeout': int(timeout)}

        if str(escape).lower() == "yes" or str(escape).lower() == "true":
            kwargs['env'] = env

        if pty_dimensions is not None:
            if LooseVersion(pexpect_obj.__version__) < LooseVersion('4.0'):
                print_warning("Setting pseudo-terminal dimensions is not supported in "
                              "pexpect versions less than 4.0(installed pexpect "
                              "version: {})".format(pexpect_obj.__version__))
            else:
                kwargs['dimensions'] = pty_dimensions

        child = pexpect_obj.spawn(command, **kwargs)
        return child

    @staticmethod
    def _send_cmd_by_type(session_object, command):
        """Determine the command type and
        send accordingly """

        if command.startswith("wctrl:"):
            command = command.split("wctrl:")[1]
            session_object.sendcontrol(command)
        else:
            session_object.sendline(command)

    @staticmethod
    def smart_analyze(prompt, testdatafile=None):
        """
            retrieve the correspond smart testdata file for smart cmd
            from either Tools/connection or testcase testdata file
            :param prompt:
                The string that will be analyzed in order to find the
                device system
            :param testdatafile:
                optional arg to provide a pre-defined device system in the
                test datafile
            :return:
                the smart datafile that contains the smart cmd to be sent
        """
        system_name = None

        if testdatafile is not None:
            # when the testdatafile is a dictionary - this happens only when
            # the testdatafile is taken from database server
            if isinstance(testdatafile, dict):
                db_td_obj = database_utils_class.\
                 create_database_connection('dataservers',
                                            testdatafile.get('td_system'))
                root = db_td_obj.get_tdblock_as_xmlobj(testdatafile)
                db_td_obj.close_connection()
            else:
                root = Utils.xml_Utils.getRoot(testdatafile)
            system_name = Utils.data_Utils._get_global_var(root, "system_name")

        #con_settings_dir = Tools.__path__[0] + os.sep + 'connection' + os.sep
        con_settings_dir = os.getenv("WAR_TOOLS_DIR") + os.sep + 'connection' + os.sep
        con_settings = con_settings_dir + "connect_settings.xml"

        if system_name is not None:
            sys_elem = Utils.xml_Utils.getElementWithTagAttribValueMatch(
                con_settings, "system", "name", system_name.text)
            if sys_elem is None or sys_elem.find("testdata") is None:
                return None
        else:
            system_list = Utils.xml_Utils.getElementListWithSpecificXpath(
                con_settings, "system[search_string]")
            for sys_elem in system_list:
                if sys_elem.find("search_string").text in prompt and \
                 sys_elem.find("testdata") is not None:
                    return con_settings_dir + sys_elem.find("testdata").text
            return None

        return con_settings_dir + sys_elem.find("testdata").text

    def send_smart_cmd(self, connect_testdata, session_object, tag_value,
                       call_system_name, pre_tag):
        """
            The beacons of Gondor are lit
            send out the smart command
            :param connect_testdata:
                the smart testdata file that contains the smart cmd
            :param session_object:
                use this pexpect object to send out command
            :param tag_value:
                specify the testdata block of commands that get sent out
            :param call_system_name:
                in order to get passed the substitutions, a system name must
                be provided
            :param pre_tag:
                Distinguish if it is a connect smart action or disconnect
                smart action
        """
        if Utils.xml_Utils.getElementWithTagAttribValueMatch(connect_testdata, "testdata", "title",
                                                             tag_value) is not None:
            print_info("**********The following command are sent as part of "
                       "the smart analysis**********")
            main_log = session_object.logfile
            if pre_tag:
                smart_log = main_log.name.replace(".log", "pre_.log")
            else:
                smart_log = main_log.name.replace(".log", "post_.log")
            session_object.logfile = open(smart_log, "ab")
            self.send_commands_from_testdata(connect_testdata, session_object,
                                             title=tag_value,
                                             system_name=call_system_name)
            session_object.logfile = main_log
            print_info("**********smart analysis finished**********")
        else:
            print_error()

    def smart_action(self, datafile, call_system_name, raw_prompt,
                     session_object, tag_value, connect_testdata=None):
        """
            entry function for sending smart command
            :param datafile:
                the testcase datafile
            :param call_system_name:
                in order to get passed the substitutions, a system name must
                be provided
            :param raw_prompt:
                The string that will be analyzed in order to find the device
                system
            :param session_object:
                use this pexpect object to send out command
            :param tag_value:
                specify the testdata block of commands that get sent out
            :param connect_testdata:
                the smart testdata file that contains the smart cmd,
                optional in here
            :return:
                the smart testdata file that contains the smart cmd
        """
        testdata, _ = Utils.data_Utils.get_td_vc(datafile, call_system_name,
                                                 None, None)
        pre_tag = False
        if connect_testdata is None:
            connect_testdata = self.smart_analyze(raw_prompt, testdata)
            pre_tag = True

        if connect_testdata is not None:
            self.send_smart_cmd(connect_testdata, session_object, tag_value,
                                call_system_name, pre_tag)
            return connect_testdata
        return None

    @staticmethod
    def get_connection_port(conn_type, inpdict):
        """Gets the port for ssh or telnet connections
        1. ssh :
            - looks if ssh_port is present in  inpdict.
            - if not checks for conn_port
            - if both not present returns None
        """
        if inpdict:
            conn_string = "{0}_port".format(conn_type)
            if conn_string in inpdict and inpdict[conn_string] is not False\
               and inpdict[conn_string] is not None:
                inpdict["port"] = inpdict["{0}_port".format(conn_type)]
            elif "conn_port" in inpdict and inpdict["conn_port"] is not False\
                 and inpdict["conn_port"] is not None:
                inpdict["port"] = inpdict["conn_port"]

        return inpdict

    @staticmethod
    def sendPing(hostname, count, fname):
        """Sends a ping command
        :Arguments:
            1. count(string) = no of pings to be sent
            2. src_iface(string) = source interface from whihc ping messages
                                    are to be sent.
            3. destip(string) = the destination ip address to ping.
            4. fname = logfile to log ping response.
        :Returns:
            status = boolean
        """
        status = False
        command = "ping -c " + count + " " + hostname + " >>" + fname
        print_debug("sendPing, cmd = '%s'" % command)

        response = os.system(command)
        if response == 0:
            print_debug("hostname : '%s' is up " % hostname)
            status = True
        print_debug("hostname : '%s' is down " % hostname)
        return status

    @staticmethod
    def sendSourcePing(count, src_iface, destip, fname):
        """Sends a source based ping command
        i.e. if multiple interfaces are configured and available,
        sends pings from the src_iface provided
        :Arguments:
            1. count(string) = no of pings to be sent
            2. src_iface(string) = source interface from whihc ping messages
                                    are to be sent.
            3. destip(string) = the destination ip address to ping.
            4. fname = logfile to log ping response.
        :Returns:
            status = boolean
        """
        status = False
        command = "ping -c " + count + " -I " + src_iface + " " + \
            destip + " >>" + fname
        print_debug("command, cmd = '%s'" % command)

        response = os.system(command)
        if response == 0:
            print_debug("hostname : '%s' is up " % destip)
            status = True
        print_debug("hostname : '%s' is down " % destip)
        return status


class ParamikoConnect(object):
    """ Class to handle SSH operations using Paramiko module """
    def __init__(self, credentials={}):
        """ Constructor

        :Arguments
            1. credentials:
                1. ip = target_system ip
                2. port = target_system ssh port
                3. username(string) = target_system username
                4. password(string) = target_system password
                5. logfile = logfile name
                6. timeout = wait for response in target_system
                7. via_ip = intermediate_system ip
                8. via_port = intermediate_system ssh port
                9. via_username(string) = intermediate_system username
                10. via_password(string) = intermediate_system password
                11. via_timeout = wait for response in intermediate_system
                12. conn_type = session type(ssh/ssh_nested/telnet)
         """

        self.paramiko = None
        self.__import_paramiko()
        self.conn_string = "".encode("utf-8")
        self.response = ""
        self.target_host = None
        self.channel = None
        self.conn_type = credentials.get('conn_type')
        if self.conn_type:
            self.conn_type = self.conn_type.upper()
        self.ip = credentials.get('ip')
        self.port = credentials.get('port')
        if self.port is not None:
            self.port = int(self.port)
        self.username = credentials.get('username', '')
        self.password = credentials.get('password', '')
        self.logfile = credentials.get('logfile')
        self.timeout = credentials.get('timeout', 60)

        if self.conn_type == "SSH_NESTED":
            self.via_ip = credentials.get('via_ip')
            self.via_port = credentials.get('via_port')
            if self.via_port is not None:
                self.via_port = int(self.via_port)
            self.via_username = credentials.get('via_username', '')
            self.via_password = credentials.get('via_password', '')
            self.via_timeout = credentials.get('via_timeout', 60)

    def __import_paramiko(self):
        """Import the paramiko module """

        try:
            import paramiko
        except ImportError:
            print_info("{0}: paramiko module is not "
                       "installed".format(os.path.abspath(__file__)))
            print_info("Warrior Framework uses paramiko module for "
                       "cli(via nested SSH) related operations")
        else:
            self.paramiko = paramiko

    def connect_ssh(self):
        """
        Initiates SSH connection to target system using paramiko module.
        For nested SSH connections, session will be established
        via intermediate system.
        """

        if self.paramiko is None:
            print_error("Paramiko is not installed, please install it")
            return

        self.port = self.port if self.port else "22"
        try:
            # for nested SSH session
            if self.conn_type == "SSH_NESTED":
                pNote("Nested SSH connection is requested, first connecting to"
                      " intermediate system - {}".format(self.via_ip))
                self.via_host = self.paramiko.SSHClient()
                self.via_host.set_missing_host_key_policy(
                    self.paramiko.AutoAddPolicy())
                self.via_host.connect(self.via_ip, port=self.via_port,
                                      username=self.via_username,
                                      password=self.via_password,
                                      timeout=self.via_timeout)

                self.via_transport = self.via_host.get_transport()

                dest_addr = (self.ip, self.port)
                local_addr = ('127.0.0.1', 22)

                self.via_channel = self.via_transport.open_channel(
                    "direct-tcpip", dest_addr, local_addr)
                pNote("Connection to intermediate system - {} "
                      "is successful".format(self.via_ip))
            else:
                self.via_channel = None

            self.target_host = self.paramiko.SSHClient()
            self.target_host.set_missing_host_key_policy(
                self.paramiko.AutoAddPolicy())

            self.target_host.connect(self.ip, port=self.port,
                                     username=self.username,
                                     password=self.password,
                                     timeout=self.timeout,
                                     sock=self.via_channel)

            if self.logfile is not None:
                # paramiko logging level is DEBUG(default)
                self.paramiko.util.log_to_file(self.logfile)
            # self.conn_string = self.target_host.get_transport().get_banner()
            # Use invoke_shell option to get conn_string value
            self.channel = self.target_host.invoke_shell()
            self.conn_string = self.channel.recv(9999)
        except Exception as exception:
            self.target_host = None
            print_exception(exception)

    def disconnect(self):
        """ Disconnects nested paramiko session """
        self.channel.close()
        self.target_host.close()
        # for nested ssh session
        if self.conn_type == "NESTED_SSH":
            self.via_host.close()

    @mocked
    def send_command(self, command, get_pty=False, *args, **kwargs):
        """ Execute the command on the remote host

        :Arguments:
            1.command = command to be executed
            2.get_pty = request a pseudo-terminal

        """

        response = ""
        status = False

        try:
            start_time = Utils.datetime_utils.get_current_timestamp()
            pNote("[{0}] COMMAND: {1}".format(start_time, command))
            stdin, stdout, stderr = self.target_host.\
                exec_command(command, get_pty=get_pty)

            end_time = Utils.datetime_utils.get_current_timestamp()
            stdin.close()

            # If a linux command fails, corresponding response will be in
            # stderr, stdout.channel.recv_exit_status() will give non-zero
            # value but stderr value is also a valid response in Warrior
            # execution.we can't fail the command execution based on
            # recv_exit_status value.
            if stdout:
                response = response + stdout.read().decode("utf-8")
            if stderr:
                response = response + stderr.read().decode("utf-8")

            pNote("[{0}] Command execution completed".format(end_time))
            print_debug("Response:\n{0}\n".format(response))
            status = True
        except Exception as exception:
            print_exception(exception)

        return status, response


class PexpectConnect(object):
    """ Class to handle SSH operations using Pexpect module """

    def __init__(self, credentials={}):
        """ Constructor

        :Arguments
            1. credentials:
                1. ip = destination ip
                2. port = ssh/telnet port
                3. username(string) = username
                4. password(string) = password
                5. logfile(string) = logfile name
                6. timeout = timeout duration
                7. prompt(string) = destination prompt
                8. conn_options = connection options
                9. custom_keystroke = keystroke(to be given
                                      after initial timeout)
                10. escape(string) = true/false(to escape color codes by
                                     setting TERM as dump)
                11. conn_type = session type(ssh/telnet)
                12. pty_dimensions(tuple) = size of the pseudo-terminal
                                            specified as a two-entry
                                            tuple(rows, columns)
         """

        self.pexpect = None
        self.__import_pexpect()
        self.conn_string = "".encode("utf-8")
        self.response = ""
        self.target_host = None
        self.channel = None
        self.conn_type = credentials.get('conn_type')
        if self.conn_type:
            self.conn_type = self.conn_type.upper()
        self.ip = credentials.get('ip')
        self.port = credentials.get('port')
        self.username = credentials.get('username', '')
        self.password = credentials.get('password', '')
        self.logfile = credentials.get('logfile')
        self.timeout = credentials.get('timeout', 60)
        self.prompt = credentials.get('prompt', '.*(%|#|\$)')
        self.conn_options = credentials.get('conn_options', '')
        self.custom_keystroke = credentials.get('custom_keystroke', '')
        self.escape = credentials.get('escape', False)
        self.pty_dimensions = credentials.get('pty_dimensions', None)
        # convert pty_dimensions value from string to tuple
        if self.pty_dimensions and isinstance(self.pty_dimensions, str):
            err_msg = ("Invalid value '{}' given for pty_dimensions "
                       "argument, it only accepts tuple value"
                       "(It will be default to None).")
            try:
                self.pty_dimensions = ast.literal_eval(self.pty_dimensions)
            except Exception:
                print_warning(err_msg.format(self.pty_dimensions))
            else:
                if not isinstance(self.pty_dimensions, tuple):
                    print_warning(err_msg.format(self.pty_dimensions))
                    self.pty_dimensions = None

    def __import_pexpect(self):
        """Import the pexpect module """

        try:
            import pexpect
        except ImportError:
            print_info("{0}: pexpect module is not "
                       "installed".format(os.path.abspath(__file__)))
            print_info("Warrior Framework uses pexpect module for "
                       "cli(SSH & telent) related operations")
        else:
            self.pexpect = pexpect

    def connect_ssh(self):
        """
        Initiates SSH connection via a specific port. Creates log file.
        """

        if self.pexpect is None:
            print_error("Pexpect is not installed, please install it")
            return

        self.target_host = None
        self.conn_string = "".encode("utf-8")
        self.port = self.port if self.port else "22"
        conn_options = "" if self.conn_options is False or \
            self.conn_options is None else self.conn_options
        custom_keystroke = "wctrl:M" if not self.custom_keystroke else \
            self.custom_keystroke
        # delete -o StrictHostKeyChecking=no and put them in conn_options
        if not conn_options or conn_options is None:
            conn_options = ""
        if not self.username:
            self.username = ""
            print_warning("Using '{0}' as username since it is not provided "
                          "in data file".format(getpass.getuser()))
        else:
            self.username += '@'
        command = 'ssh -p {0} {1}{2} {3}'.format(self.port, self.username,
                                                 self.ip, conn_options)
        # command = ('ssh -p '+ port + ' ' + username + '@' + ip)
        print_debug("connectSSH: cmd = %s" % command)
        child = WarriorCli.pexpect_spawn_with_env(self.pexpect, command,
                                                  self.timeout,
                                                  escape=self.escape,
                                                  env={"TERM": "dumb"},
                                                  pty_dimensions=self.pty_dimensions)

        child.logfile = sys.stdout

        if self.logfile is not None:
            try:
                fdobj = open(self.logfile, "ab")
                if fdobj:
                    child.logfile = fdobj
            except Exception as exception:
                print_exception(exception)

        try:
            flag = True
            child.setecho(False)
            child.delaybeforesend = .5
            while True:
                result = child.expect(["(yes/no)", self.prompt,
                                       '.*(?i)password:.*',
                                       ".*(?i)(user(name)?:|login:) *$",
                                       self.pexpect.EOF, self.pexpect.TIMEOUT,
                                       '.*(?i)remote host identification has '
                                       'changed.*'])
                if result == 0:
                    child.sendline('yes')
                elif result == 1:
                    self.target_host = child
                    self.conn_string = self.conn_string + child.before + \
                        child.after
                    break
                elif result == 2:
                    child.sendline(self.password)
                    self.conn_string = self.conn_string + child.before + \
                        child.after
                elif result == 3:
                    child.sendline(self.username)
                elif result == 4:
                    pNote("Connection failed: {0}, with the system response: "
                          "{1}".format(command, child.before), "error")
                    break
                elif result == 5:
                    # Some terminal expect specific keystroke before showing
                    # login prompt
                    if flag is True:
                        pNote("Initial timeout occur, "
                              "sending custom_keystroke")
                        WarriorCli._send_cmd_by_type(child, custom_keystroke)
                        flag = False
                        continue
                    pNote("Connection timed out: {0}, expected prompt: {1} "
                          "is not found in the system response: {2}"
                          .format(command, self.prompt, child.before), "error")
                    break
                elif result == 6:
                    cmd = "ssh-keygen -R " + self.ip if self.port == '22' else\
                          "ssh-keygen -R " + "[" + self.ip + "]:" + self.port
                    print_debug("SSH Host Key is changed - Remove it from "
                                "known_hosts file : cmd = %s" % cmd)
                    subprocess.call(cmd, shell=True)
                    child = WarriorCli.pexpect_spawn_with_env(self.pexpect, command,
                                                              self.timeout,
                                                              escape=self.escape,
                                                              env={"TERM": "dumb"},
                                                              pty_dimensions=self.pty_dimensions)
                    print_debug("ReconnectSSH: cmd = %s" % command)
        except Exception as exception:
            print_exception(exception)

    def connect_telnet(self):
        """
        Initiates Telnet connection via a specific port. Creates log file.
        """

        self.target_host = None
        self.conn_string = "".encode("utf-8")
        self.port = self.port if self.port else "23"
        conn_options = "" if self.conn_options is False or \
            self.conn_options is None else self.conn_options
        custom_keystroke = "wctrl:M" if not self.custom_keystroke else \
            self.custom_keystroke
        print_debug("timeout is: %s" % self.timeout)
        print_debug("port num is: %s" % self.port)
        command = ('telnet ' + self.ip + ' ' + self.port)
        if not conn_options or conn_options is None:
            conn_options = ""
        command = command + str(conn_options)
        print_debug("connectTelnet cmd = %s" % command)

        child = WarriorCli.pexpect_spawn_with_env(self.pexpect, command,
                                                  self.timeout,
                                                  env={"TERM": "dumb"},
                                                  escape=self.escape,
                                                  pty_dimensions=self.pty_dimensions)

        try:
            child.logfile = open(self.logfile, "ab")
        except Exception:
            child.logfile = None

        try:
            flag = True
            child.setecho(False)
            child.delaybeforesend = .5
            while True:
                result = child.expect([self.prompt, '.*(?i)password:.*',
                                       ".*(?i)(user(name)?:|login:) *$",
                                       self.pexpect.EOF,
                                       self.pexpect.TIMEOUT])
                if result == 0:
                    self.target_host = child
                    self.conn_string = self.conn_string + child.before + \
                        child.after
                    break
                elif result == 1:
                    child.sendline(self.password)
                    self.conn_string = self.conn_string + child.before + \
                        child.after
                elif result == 2:
                    child.sendline(self.username)
                elif result == 3:
                    pNote("Connection failed: {0}, with the system response: "
                          "{1}".format(command, child.before), "error")
                    break
                elif result == 4:
                    # timed out tryonce with Enter has some terminal expects it
                    if flag is True:
                        pNote("Initial timeout occur, "
                              "sending custom_keystroke")
                        WarriorCli._send_cmd_by_type(child, custom_keystroke)
                        flag = False
                        continue
                    pNote("Connection timed out: {0}, expected prompt: {1} "
                          "is not found in the system response: {2}"
                          .format(command, self.prompt, child.before), "error")
                    break
        except Exception as exception:
            print_error(" ! could not connect to %s...check logs" % self.ip)
            print_exception(exception)

    def disconnect(self):
        """
        Disconnects a pexpect session
        """
        if self.target_host.isalive():
            if self.target_host.ignore_sighup:
                self.target_host.ignore_sighup = False
            self.target_host.close()

    def disconnect_telnet(self):
        """
        Disconnects a telnet session
        """
        if self.target_host.isalive():
            time.sleep(2)
            self.target_host.sendcontrol(']')
            time.sleep(2)
            self.target_host.expect('telnet> ')
            time.sleep(2)
            self.target_host.sendline('q')
            time.sleep(2)
            self.target_host.close()

    @mocked
    def send_command(self, command, start_prompt, end_prompt,
                     timeout=60, *args, **kwargs):
        """
        Send an command to pexpect session object and returns
        the status of the command sent
        - Checks for the availability of the start_prompt.
        - if start prompt was available sends the command
        - if failure response is not None and failure response found in
        response then returns False.
        - else if failure response was not found
        and end prompt also not found returns False.
        - else if failure response was not found and end prompt found,
        then returns true.
        """
        tmout = {None: 60, "": 60, "none": 60}.get(timeout,
                                                   str(timeout).lower())
        self.target_host.timeout = int(tmout)
        pNote("Command timeout: {0}".format(self.target_host.timeout))
        response = ""
        msg = ""
        end_time = False
        status = False
        cmd_timedout = False
        # time_format = "%Y-%b-%d %H:%M:%S"
        try:
            boolprompt = self.target_host.expect(start_prompt)
        except Exception as exception:
            pNote("Could not find the start_prompt '{0}'!! exiting!!".
                  format(str(start_prompt)), "error")
            boolprompt = -1
        if boolprompt == 0:
            start_time = Utils.datetime_utils.get_current_timestamp()
            if kwargs.get("log", "true") != "false":
                pNote("[{0}] COMMAND: {1}".format(start_time, command))
            WarriorCli._send_cmd_by_type(self.target_host, command)

            if current_process().name == 'MainProcess':
                pass
            else:
                time.sleep(2)

            try:
                while True:
                    if kwargs.get("sleep_before", None):
                        pNote("Sleeping for {0} seconds before match" \
                            .format(kwargs["sleep_before"]))
                        time.sleep(int(kwargs["sleep_before"]))
                    result = self.target_host.expect([end_prompt,
                                                      self.pexpect.EOF,
                                                      self.pexpect.TIMEOUT]) \
                                                      if end_prompt else -1
                    end_time = Utils.datetime_utils.get_current_timestamp()
                    if result == 0:
                        curr_time = Utils.datetime_utils.\
                            get_current_timestamp()
                        msg1 = "[{0}] Command completed successfully". \
                            format(end_time)
                        msg2 = "[{0}] Found end prompt '{1}' after command" \
                            "had timed out".format(curr_time, end_prompt)
                        status = {True: "ERROR", False: True}.get(cmd_timedout)
                        msg = {True: msg2, False: msg1}.get(cmd_timedout)
                        break
                    elif result == -1:
                        pNote("[{0}] end prompt not provided".format(end_time),
                              "error")
                        status = "ERROR"
                        break
                    elif result == 1:
                        msg = "[{0}] EOF encountered".format(end_time)
                        status = "ERROR"
                        break
                    elif result == 2:
                        tmsg1 = "[{0}] Command timed out with {1} seconds, command will be " \
                            "marked as error".format(end_time, timeout)
                        tmsg2 = "As a best effort, warrior will wait 60 more seconds to get end " \
                            "prompt '{0}'".format(end_prompt)
                        tmsg3 = "Irrespective of whether end prompt is " \
                            "received or not command will be marked as error" \
                            " because command had timed out once."
                        if not cmd_timedout:
                            self.target_host.timeout = 1
                            pNote(tmsg1, "debug")
                            pNote(tmsg2, "debug")
                            pNote(tmsg3, "debug")
                            tstamp = Utils.datetime_utils.\
                                get_current_timestamp()
                        cmd_timedout = True
                        status = "ERROR"
                        tdelta = Utils.datetime_utils.get_time_delta(tstamp)
                        if int(tdelta) >= 60:
                            msg = "[{0}] Did not find end prompt '{1}' even " \
                                "after 60 seconds post command time out". \
                                format(Utils.datetime_utils.
                                       get_current_timestamp(),
                                       end_prompt)
                            break
                        else:
                            continue
            except Exception as exception:
                print_exception(exception)
            else:
                response = self.target_host.before.decode('utf-8')
                # When the command gets timed out, the pexpect.TIMEOUT exception
                # will be raised and it is set to the after property of spawn object
                step_num = data_repository['step_num']
                if self.target_host.after == self.pexpect.TIMEOUT:
                    pNote("EXCEPTION !! Command Timed Out", 'error')
                    data_repository['step_%s_errormessage' % step_num] = "command timed out while executing command: {}".format(command.split(":")[0])
                elif self.target_host.after == self.pexpect.EOF:
                    pNote("EXCEPTION !! Device unresponsive", 'error')
                    data_repository['step_%s_errormessage' % step_num] = "device unresponsive while executing command: {}".format(command.split(":")[0])
                else:
                    try:
                        response = response + self.target_host.after.decode('utf-8')
                    except:
                        pNote("EXCEPTION !! Cannot decode response", 'error')
                        data_repository['step_%s_errormessage' % step_num] = "Response decode error for command: {}".format(command.split(":")[0])
                if kwargs.get("log", "true") != "false":
                    print_info("RESPONSE:\n{0}\n".format(response))
                pNote(msg, "debug")
                if status is True:
                    duration = Utils.datetime_utils.get_time_delta(start_time,
                                                                   end_time)
                    data_repository.update({"command_duration" : duration})
        return status, response



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

""" Keywords related to File operations """

from warrior.Framework import Utils
from warriornetwork.ClassUtils.WNetwork.file_ops import FileOps
class FileOpsActions(object):
    """File operations keyword class """

    def __init__(self):
        """ Constructor """
        self.resultfile = Utils.config_Utils.resultfile
        self.datafile = Utils.config_Utils.datafile
        self.logsdir = Utils.config_Utils.logsdir
        self.filename = Utils.config_Utils.filename
        self.logfile = Utils.config_Utils.logfile
        self.fileops = FileOps()

    def ftp_from_remotehost(self, system_name, session_name=None,
                            dest_system=None, ftp_operation="both",
                            ip_type="ip"):
        """ftp(test both put and get).This keyword can be used to transfer
        file from source _system to destination system or vice versa.
        It checks the size after transfer in both get and put.

        :Datafile usage:
            Tags or attributes to be used in input datafile for the system
            or subsystem.If both tag and attribute is provided the attribute
            will be used.

            1. prompt = prompt of the source system
            2. filepath = file with filepath in source system(used for put)
            3. ip = ipv4 address or defaulted to lcn ip
            4. ipv6 = ipv6 address or defaulted to lcn ipv6
            5. dns = dns name
            6. lmp_ip = ipv4 address of lmp port of dut
            7. lmp_ipv6 = ipv6 address of lmp port of dut
            8. username = username of the dest system
            9. password = password of the dest system
            10. filepath = file with filepath in destination system \
                           (used for get)
            note: one of <ip>,<ipv6>,<dns>,<lmp_ip>,<lmp_ipv6> tags specified in
                  ip_type argument is sufficient to be present in inputdatafile

        :Arguments:
            1. system_name(string)  = name of the Linux machine on which\
                                      to execute
            2. session_name(string) = name of the session
            3. dest_system(string) = names of the destination system
            4. ftp_operation(string) = get/put/both
            6. ip_type(string) = iptype of the dest system through \
                                 which it needs to be connected.
                                 needs to be one of \
                                 (ip/ipv4/dns/lmp_ip/lmp_ipv6).It has to be \
                                 present in the input data file.

        :Returns:
            1. bool (True/False)

        """
        wdesc = "ftp file from and to dest_system"
        Utils.testcase_Utils.pSubStep(wdesc)
        Utils.testcase_Utils.pNote(system_name)
        Utils.testcase_Utils.pNote(self.datafile)

        keys_for_credentials = [ip_type, "username",
                                "password", "filepath"]

        credentials_dest = Utils.data_Utils.get_credentials(self.datafile,
                                                            dest_system,
                                                            keys_for_credentials
                                                           )
        credentials = Utils.data_Utils.get_credentials(self.datafile,
                                                       system_name,
                                                       ["filepath",
                                                        "prompt"])

        session_id = Utils.data_Utils.get_session_id(system_name, session_name)
        session_object = Utils.data_Utils.get_object_from_datarepository(session_id)
        if session_object:
            command_status = self.fileops.ftp_from_remotehost(session_object, ip_type,
                                                              ftp_operation,
                                                              credentials["filepath"],
                                                              credentials["prompt"],
                                                              credentials_dest[ip_type],
                                                              credentials_dest["username"],
                                                              credentials_dest["password"],
                                                              credentials_dest["filepath"])
            status = command_status
        else:
            Utils.testcase_Utils.pNote(("%s-%s is not available for use" %
                                        system_name, session_name), "warning")
            status = False
        Utils.testcase_Utils.report_substep_status(status)
        return status

    def sftp_from_remotehost(self, system_name, session_name=None,
                             dest_system=None, sftp_operation="both",
                             port="1022", ip_type="ip"):
        """sftp(test both put and get).This keyword can be used to transfer
        file from source _system to destination system or vice versa.
        It checks the size after transfer in both get and put.

        :Datafile usage:
            Tags or attributes to be used in input datafile for the system
            or subsystem.If both tag and attribute is provided the attribute
            will be used.

            1. prompt = prompt of the source system
            2. filepath = file with filepath in source system(used for put)
            3. ip = ipv4 address or defaulted to lcn ip
            4. ipv6 = ipv6 address or defaulted to lcn ipv6
            5. dns = dns name
            6. lmp_ip = ipv4 address of lmp port of dut
            7. lmp_ipv6 = ipv6 address of lmp port of dut
            8. username = username of the dest system
            9. password = password of the dest system
            10. filepath = file with filepath in destination system(used for get)
            note: one of <ip>,<ipv6>,<dns>,<lmp_ip>,<lmp_ipv6> tags specified in
                  ip_type argument is sufficient to be present in inputdatafile

        :Arguments:
            1. system_name(string)  = name of the Linux machine on which\
                                      to execute
            2. session_name(string) = name of the session
            3. dest_system(string) = names of the destination system
            4. ftp_operation(string) = get/put/both
            5. ip_type(string) = iptype of the dest system through \
                                 which it needs to be connected.
                                 needs to be one of \
                                 (ip/ipv4/dns/lmp_ip/lmp_ipv6).It has to be \
                                 present in the input data file.

        :Returns:
            1. bool (True/False)

        """
        wdesc = "sftp file from and to dest_system"
        Utils.testcase_Utils.pSubStep(wdesc)
        Utils.testcase_Utils.pNote(system_name)
        Utils.testcase_Utils.pNote(self.datafile)

        keys_for_credentials = [ip_type, "username",
                                "password", "filepath"]

        credentials_dest = Utils.data_Utils.get_credentials(self.datafile,
                                                            dest_system,
                                                            keys_for_credentials
                                                           )

        credentials = Utils.data_Utils.get_credentials(self.datafile,
                                                       system_name,
                                                       ["filepath",
                                                        "prompt"])

        session_id = Utils.data_Utils.get_session_id(system_name, session_name)
        session_object = Utils.data_Utils.get_object_from_datarepository(session_id)
        if session_object:
            command_status = self.fileops.sftp_from_remotehost(session_object,
                                                               ip_type, sftp_operation,
                                                               port, credentials["filepath"],
                                                               credentials["prompt"],
                                                               credentials_dest[ip_type],
                                                               credentials_dest["username"],
                                                               credentials_dest["password"],
                                                               credentials_dest["filepath"])
            status = command_status
        else:
            Utils.testcase_Utils.pNote(("%s-%s is not available for use" %
                                        system_name, session_name), "warning")
            status = False
        Utils.testcase_Utils.report_substep_status(status)
        return status



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

"""
Module to collect the response on a connected session
as a separate thread
"""

import threading
import time
import sys
import os

from warriornetwork.ClassUtils.WNetwork.base_class import Base
from warrior.Framework.Utils.print_Utils import print_exception, print_error, \
 print_info, print_warning

try:
    if 'linux' in sys.platform:
        import pexpect
except Exception:
    print_info("{0}: {1} module is not installed".format(
     os.path.abspath(__file__), 'pexpect'))


class ThreadedLog(Base):
    """
    Collect the response from a connected session
    as a separate thread
    """

    def __init__(self, *args, **kwargs):
        """
        Constructor
        """
        super(ThreadedLog, self).__init__(*args, **kwargs)
        self.data = None
        self.stop_thread_flag = False
        self.current_thread = None
        self.stop_thread_err_msg = ""

    def start_thread(self, session):
        """
        Starts a thread using the value self.function
        if self.function is false throws an error that
        self.function is not available.
        """
        value = False
        try:
            if session:
                self.current_thread = threading.Thread(target=self.collect_log,
                                                       args=(session,))
                self.current_thread.start()
            else:
                print_error("Need a valid session to start collecting logs")
        except Exception as exception:
            print_exception(exception)
        else:
            value = True
        return value

    def stop_thread(self):
        """
        stops the thread by setting the self.stop_thread as True
        """
        self.stop_thread_flag = True
        self.stop_thread_err_msg = ""
        return

    def thread_status(self):
        """
        Returns the status of the thread
        """
        return self.current_thread.isAlive()

    def join_thread(self, timeout=30, retry=1):
        """
        Call join method of Thread class to block caller thread until
        the current_thread terminates or until the retry counter expires
        """
        if self.current_thread is not None and retry > 0:
            try:
                self.current_thread.join(timeout)
            except Exception as err_msg:
                print_warning("Joining thread failed : {}".format(err_msg))
                self.stop_thread_err_msg += str(err_msg) + "\n"

            retry -= 1
            if self.thread_status() is True and retry > 0:
                print_warning("Thread is still alive, retry joining thread."
                              "Remaining retrial attempts: {}".format(retry))
                self.join_thread(timeout, retry)

    def collect_log(self, session):
        """
        Collects the response from a connected session
        till the tread is stopped

        This function currently collects response from
        a connected pexpect spawn object using the
        pexpect read_nonblocking method
        """
        response = " "
        while not self.stop_thread_flag:
            try:
                # default timeout for pexpect-spawn object is 30s
                string = session.read_nonblocking(1024, timeout=30)
                if isinstance(string, bytes):
                    string = string.decode("utf-8")
                response = response + string
                time.sleep(0.5)
                self.data = response
            # continue reading data from 'session' until the thread is stopped
            except pexpect.TIMEOUT:
                continue
            except Exception as exception:
                print_exception(exception)
                break


'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''


from warriornetwork.ClassUtils.WNetwork.connection import Connection
from warriornetwork.ClassUtils.WNetwork.diagnostics import Diag
from warriornetwork.ClassUtils.WNetwork.file_ops import FileOps
from warriornetwork.ClassUtils.WNetwork.base_class import Base


class Network(Connection, Diag, FileOps, Base):
    """ Warrior Network module
    This is class that inherits all other
    classes in the Network package.
    Instance of this class may be used to invoke the
    methods of all the other classes in this package.
    While using the instance of this class, if the Base
    classes have same method name, then the method will be executed
    based on python's inheritance MRO (Method Resolution Order)
    Execute 'Network.__mro__' to find out the current MRO
    Order is from left to right of the MRO.
    """

    def __init__(self, *args, **kwargs):
        """ Constructor"""

        super(Network, self).__init__(*args, **kwargs)



'''
Copyright 2017, Fujitsu Network Communications, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

""" Keywords related to Network Diagnostics operations """

from warrior.Framework import Utils
from warriornetwork.ClassUtils.WNetwork.diagnostics import Diag
class DiagActions(object):
    """Diagnostics keyword class """

    def __init__(self):
        """ Constructor """
        self.resultfile = Utils.config_Utils.resultfile
        self.datafile = Utils.config_Utils.datafile
        self.logsdir = Utils.config_Utils.logsdir
        self.filename = Utils.config_Utils.filename
        self.logfile = Utils.config_Utils.logfile
        self.diag = Diag()

    def ping_from_remotehost(self, system_name, session_name=None,
                             dest_system=None, ip_type="ip", count="5"):
        """This keyword will use connection session available in source system
        (provided in system name) and will ping from source system
        to destination system

        :Datafile usuage:
            Tags or attributes to be used in input datafile for the system or
            subsystem.If both tag and attribute is provided the attribute
            will be used.
            1. prompt   = prompt of the source system
            2. ip       = ipv4 address or defaulted to lcn ip
            3. ipv6     = ipv6 address or defaulted to lcn ipv6
            4. dns      = dns name
            5. lmp_ip   = ipv4 address of lmp port of dut
            6. lmp_ipv6 = ipv6 address of lmp port of dut
            note: one of <ip>,<ipv6>,<dns>,<lmp_ip>,<lmp_ipv6> tags specified in
                  ip_type argument is sufficient to be present in inputdatafile

        :Arguments:
            1. system_name(string)  = name of the Linux machine on which to \
                                      execute
            2. session_name(string) = name of the session
            3. dest_system(string) = names of the destination system
            4. ip_type(string) = iptype of the dest system through \
                                 which it needs to be connected.
                                 needs to be one of \
                                 (ip/ipv4/dns/lmp_ip/lmp_ipv6).It has to be \
                                 present in the input data file.
            5. count(string) = count argument to be supplied to the ping

        :Returns:
            1. bool (True/False)

        """
        wdesc = "ping  to destination system"
        Utils.testcase_Utils.pSubStep(wdesc)
        Utils.testcase_Utils.pNote(system_name)
        Utils.testcase_Utils.pNote(self.datafile)

        credentials_dest = Utils.data_Utils.get_credentials(self.datafile,
                                                            dest_system,
                                                            [ip_type])
        credentials = Utils.data_Utils.get_credentials(self.datafile,
                                                       system_name,
                                                       ["prompt"])

        session_id = Utils.data_Utils.get_session_id(system_name, session_name)
        session_object = Utils.data_Utils.get_object_from_datarepository(session_id)
        if session_object:
            command_status = self.diag.ping_from_remotehost(session_object,
                                                            ip_type,
                                                            credentials_dest[ip_type],
                                                            credentials["prompt"], count)
            status = command_status
        else:
            Utils.testcase_Utils.pNote(("%s-%s is not available for use" %
                                        system_name, session_name), "warning")
            status = False
        Utils.testcase_Utils.report_substep_status(status)
        return status

    def traceroute_from_remotehost(self, system_name, session_name=None,
                                   dest_system=None, ip_type="ip"):
        """This keyword will use connection session available in source system
        (provided in system name) and  will execute traceroute
        from source system to destination system

        :Datafile usuage:
            Tags or attributes to be used in input datafile for the system or
            subsystem.If both tag and attribute is provided the attribute will
            be used.
            1. prompt   = prompt of the source system
            2. ip       = ipv4 address or defaulted to lcn ip
            3. ipv6     = ipv6 address or defaulted to lcn ipv6
            4. dns      = dns name
            5. lmp_ip   = ipv4 address of lmp port of dut
            6. lmp_ipv6 = ipv6 address of lmp port of dut
            note: one of <ip>,<ipv6>,<dns>,<lmp_ip>,<lmp_ipv6> tags specified in
                  ip_type argument is sufficient to be present in inputdatafile

        :Arguments:
            1. system_name(string) = name of the Linux machine on which to \
                                     execute
            2. session_name(string) = name of the session
            3. dest_system(string) = names of the destination system
            4. ip_type(string) = iptype of the dest system through \
                                 which it needs to be connected.
                                 needs to be one of \
                                 (ip/ipv4/dns/lmp_ip/lmp_ipv6).It has to be \
                                 present in the input data file.

        :Returns:
            1. bool (True/False)

        """
        wdesc = "traceroute to dest system"
        Utils.testcase_Utils.pSubStep(wdesc)
        Utils.testcase_Utils.pNote(system_name)
        Utils.testcase_Utils.pNote(self.datafile)

        credentials_dest = Utils.data_Utils.get_credentials(self.datafile,
                                                            dest_system,
                                                            [ip_type])
        credentials = Utils.data_Utils.get_credentials(self.datafile,
                                                       system_name,
                                                       ["prompt"])
        session_id = Utils.data_Utils.get_session_id(system_name, session_name)
        session_object = Utils.data_Utils.get_object_from_datarepository(session_id)
        if session_object:
            command_status = self.diag.traceroute_from_remotehost(session_object,
                                                                  ip_type,
                                                                  credentials_dest[ip_type],
                                                                  credentials["prompt"])
            status = command_status
        else:
            Utils.testcase_Utils.pNote(("%s-%s is not available for use" %
                                        system_name, session_name), "warning")
            status = False
        Utils.testcase_Utils.report_substep_status(status)
        return status
